
######################################################################
#
# connect panel and connection menubar events 
#
######################################################################

######################################################################
#
# user clicks on a record in the connect window 
#
proc connect_panel_click { list y } {
	global snbVar snbConst mySearch myConnect myChain

	# no items to select
	if { $mySearch(searchNum) == 0 } { return }

	# find out which item was selected
	set snbVar(panel,connectSelect) [$mySearch(items) nearest $y]

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)

	set cix $mySearch($snbVar(panel,connectSelect),cix)
	set ref0 $myConnect($cix,ref,0)
	set ref1 $myConnect($cix,ref,1)
	set rule $myConnect($cix,rule)
#	usnb_debug "Click connection panel item $ref0-$ref1 $rule"
	panel_state_buttons "add draw"

	# current chain is empty
	if { $numConnect == 0 } {
		set snbVar(panel,addToChain) 0 
		return
	}

	# we can append this connection to the first connection of current chain 
	if { $ref1 == $myConnect($myChain($cur,1,cix),ref,0) } {
		set snbVar(panel,addToChain) 0
		return
	}

	# we can append this connection to the last connection of current chain 
	if { $ref0 == $myConnect($myChain($cur,$numConnect,cix),ref,1) } {
		set snbVar(panel,addToChain) [expr $numConnect + 1]
		return
	}

	# check for "replace" or delete at first connection of the chain 
	if { $ref1 == $myConnect($myChain($cur,1,cix),ref,1) } {
		set snbVar(panel,addToChain) 1
		return
	}

	# check for "replace" or delete at last connection of the chain 
	if { $ref0 == $myConnect($myChain($cur,$numConnect,cix),ref,0) } {
		set snbVar(panel,addToChain) $numConnect 
		return
	}

	# if any of the above is not matched, "add to chain" is not allowed
	panel_state_buttons "draw"
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# init connectivity panel 
#
proc connect_panel_init { } {
	global snbVar snbConst mySearch

	set snbVar(panel,connectSelect) -1
	$mySearch(items) delete 0 end 
	set mySearch(searchNum) 0 
}

######################################################################
#
# Update connection panel 
#
proc connect_panel_update { } {
	global snbVar snbConst mySearch

	set snbVar(panel,connectSelect) -1
	$mySearch(items) delete 0 end 
	set count 0 
	foreach item $mySearch(foundList) {
		set fit [lindex $item 1]
		set mySearch($count,cix) [lindex $item 2]
		set mySearch($count,pix) [lindex $item 3]
		set line [connect_format_line $count $fit]
		$mySearch(items) insert end $line
		incr count
	}
	set mySearch(searchNum) $count
}

######################################################################
#
# Create one entry line for the connection panel 
#
proc connect_format_line { item fit } {
	global snbVar snbConst myConnect myChain mySearch mySpectra

	set cix $mySearch($item,cix)
	set pix $mySearch($item,pix)
	set rule $myConnect($cix,rule)
	set ref0 $myConnect($cix,ref,0)
	set ref1 $myConnect($cix,ref,1)

	# compute status field which shows state of the connection
	set a1 "-" 
	set a2 "-"
	if { [llength $myChain($ref0,chainId)] > 1 } { set a1 "C" }
	if { [llength $myChain($ref1,chainId)] > 1 } { set a2 "C" }
	set ids [lrange $myChain($ref0,chainId) 1 end]
	foreach i $ids {
		if { $myChain($i,aaIx) >= 0 } {
			set a1 "A"
		}
	}
	set ids [lrange $myChain($ref1,chainId) 1 end]
	foreach i $ids {
		if { $myChain($i,aaIx) >= 0 } {
			set a2 "A"
		}
	}

	# determine if this item matches the selected item in the chains panel
	set mark "  "
	set cur $myChain(current)
	set pick $snbVar(panel,chainSelect)
	if { $pick >= 1 && $pick <= $myChain($cur,nConnect)} {
		if { $myChain($cur,$pick,cix) == $cix } {
			if { $myChain($cur,$pick,pix) == $pix} {
				set mark $snbConst(panelSelect)
			}
		}
	}

	# get probability score of connection
	# if no record, then adjust chain panel record appropriately
	set iProb $mySpectra(connect,PROB)
	if {[ catch { set prob $myConnect($cix,$pix,$iProb) } err]} {
		set prob 0.0
		set fit 0
		set pix 0
		set mark $snbConst(panelSelect)
	}

	set pNum $myConnect($cix,pNum)
	set line [format "%2s %1d %3.2f %3s-%-3s %3s %2d/%-2d %1s%1s" \
		$mark $fit $prob $ref0 $ref1 $rule $pix $pNum $a1 $a2]
	return $line
}

######################################################################
#
# Return item value from connections database 
#
proc connect_get_val { cix pix item } {
	global snbVar snbConst myConnect

	set rule  $myConnect($cix,rule)
	set ref0 $myConnect($cix,ref,0)
	set ref1 $myConnect($cix,ref,1)

	if ![info exists myConnect($cix,$pix,$item)] {
		usnb_err "No connection item for $cix,$pix,$item\n rule=$rule, ref0=$ref0 ref1=$ref1."
	}

	set val $myConnect($cix,$pix,$item)
	return $val
}

######################################################################
#
# create connection files 
#
proc connect_make_data { } {
	global snbVar snbConst mySeq mySpectra

	if { $snbVar(parm,calcConnect) == 0 } {
		usnb_log all "User specified to not generate default connection files"
		return
	}

	# make sure we have scripts to create connect files
   	set progStr "$mySpectra(fn,rulesdir)/$snbVar(parm,c_prog).*"
	if {[ catch { set progFiles [glob $progStr] } err]} {
		usnb_err "Cannot find any connections run files in '$progStr'."
	}

	# update each connection file
	set num [llength $progFiles]
	usnb_log all " "
	usnb_log all "Connectivity files do not exist or peaklists have changed" 
	usnb_log all "Creating $num connections files."
	foreach i $progFiles {
		set ff [file tail $i]

		# do not calculate patterns which are not possible in the sequence.
		set pType dgg
		if { [string first $pType $ff] >= 0 } {
			if { $mySeq($pType,num) < 1 } {
				set fTail "$snbVar(parm,c_out).$snbVar(parm,c_prog).$pType"
				set fn $snbVar(fn,session)/$fTail
				if {[ catch { set fout [open $fn w ] } err]} {
					usnb_err "Cannot open file $fn"
				}
				puts $fout "# Connections not applicable for this sequence "
				usnb_log all "  Executing $ff... not applicable, no gly-gly in sequence"
				close $fout
				continue
			}
		}
		usnb_log all "  Executing $ff..."
		sys_run_connect $i $mySpectra(fn,rulesdir) $snbVar(fn,session) $mySpectra(xpk,outFnames)
	}
}

######################################################################
#
# create main connectivity reference record
#
proc connect_create_ref { cix rule ref0 ref1 } {
	global snbVar snbConst myConnect

	if { $cix > $snbConst(MAX_CONNECT) } {
		 usnb_err "Too many connections... the maximum for snbConst(MAX_CONNECT) is $snbConst(MAX_CONNECT)."
	}

	set myConnect(index,$rule,$ref0,$ref1) $cix
	set myConnect($cix,rule) $rule
	set myConnect($cix,ref,0) $ref0
	set myConnect($cix,ref,1) $ref1
	set myConnect($cix,pNum) 0
}

######################################################################
#
# create specific connectivity record for a given reference pair
#
proc connect_create_item { cix line status } {
	global snbVar snbConst myConnect mySpectra

	incr myConnect($cix,pNum)
	set pix $myConnect($cix,pNum)

	# save the individual connection fields
	set myConnect($cix,$pix,cStatus) $status
	set field 0
	foreach item $line {
		set myConnect($cix,$pix,$field) $item
		incr field
	}
	for {set i $field} {$i < $mySpectra(connect,FIELDNUM)} {incr i} {
		set myConnect($cix,$pix,$i) $snbConst(NO_VAL) 
	}
}
