######################################################################
#
# Routines for the FILE menu item
#
######################################################################

######################################################################
#
# Import connectivities window 
#
proc file_import_connect { } {
	global snbVar snbConst

	set win [usnb_show_win import "Import Custom Connectivity Files"]
	if { $win == "FALSE" } { return }

	set f $win.frame
	pack [frame $f] -fill x -expand true -anchor n -padx 20

	message $f.intro -text \
		"Enter the pathname to the connectivity files you wish to import.\
		Wildcard characters are acceptable. Files will be copied to the\
		snb.out directory with the file prefix '$snbVar(parm,c_out)' added."
	pack $f.intro 
	pack [frame $f.fr] -pady 15 -fill both -expand true

	u_file_entry $f.fr.dir "Connectivity file(s):" \
		32 snbVar(fn,connect,importFname) 36 $snbVar(snbwin,padx) { } 1 0
    pack $f.fr.dir
	pack [frame $f.fr.pat] -anchor w -pady 5

	# button widgets
	pack [frame $f.buttons] -pady 15 
	pack [button $f.buttons.save -text "ok" \
		-padx $snbVar(snbwin,padx) \
		-command "do_import_connect $f; usnb_kill_win_nosave $win"] -side left
	pack [button $f.buttons.cancel -text "Dismiss" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_nosave $win" ] -side left
}


######################################################################
#
# Read option files and exit 
#
proc do_import_connect { f } {
	global snbVar snbConst

	set seek [string trim $snbVar(fn,connect,importFname)]
	if { [string length $seek] == 0 } {
		usnb_err "Bad connectivity filename, please try again."
	}

	set input [glob -nocomplain $seek]
	set count 0
	foreach fn $input {
		if ![file readable $fn] {
			usnb_err "File not readable: '$fn'."
		}

		# see if we have to add file prefix
		set fname [file tail $fn]
		set len [expr [string length $snbVar(parm,c_out)] - 1]
		if {[string range $fname 0 $len] == $snbVar(parm,c_out)} {
			set outfn $snbVar(fn,session)/$fname
		} else {
			set outfn $snbVar(fn,session)/$snbVar(parm,c_out).$fname
		}
		sys_copy_file $fn $outfn
		usnb_log all "Import connection file '$fname' into '$outfn'"
		incr count
	}

	if {$count == 0} {
		usnb_err "Expecting to find connectivity files but none found."
	} else {
		snb_read_data $snbConst(E_PEAKCON)
	}
}

######################################################################
#
# user exits smartnotebook using QUIT menu item 
#
proc file_snb_exit { } {
	global snbVar snbConst

	usnb_log all "Saving current configuration, exiting smartnotebook"
	usnb_destroy_windows
}
