
######################################################################
#
# compute fitting scor for adding a new connection 
#
######################################################################

######################################################################
#
# Can this connectivity be appended to the start of current chain such
# that a valid assignment still exists
#
proc fit_to_start_chain { cix } {
	global snbVar snbConst mySeq myChain mySearch myConnect mySpectra

	# exit if the overlapping connection and chain data disagree
	set nConnect $myChain($myChain(current),nConnect)
	set pix [ test_compatible $cix 0 1 1 ]
	if { $pix == -1 } { return }
	set myConnect($cix,pix) $pix

	# keep track of best score of the valid possible assignments
	set bestStat $snbConst(MISSING_STAT) 

	# Look through the entire sequence.
	set nTest [expr $mySeq(seqLen) - $nConnect]
	for {set aaIx 2} {$aaIx <= $nTest} {incr aaIx} {
		set location [expr $aaIx - 1]

		# Only consider start of possible assignment chains
		if { $mySeq($aaIx,testAssign) == "START_POSSIBLE" } {
			set scor [ fit_test_residue $location $cix 1 0 ]
			if { $scor < $bestStat } {
				set bestStat $scor
			}
		}
	}

	# add to our search list
	if { $bestStat < $snbVar(bestfit,sdevTerm) } {
		set iProb $mySpectra(connect,PROB)
		set fit [ calc_fit_scor $bestStat ]
		if {[ catch { set prob $myConnect($cix,$pix,$iProb) } err]} {
			set prob 0.0
		}
		lappend mySearch(foundList) "$prob $fit $cix $pix" 
	} 
}

######################################################################
#
# Can this connectivity be appended to the end of current chain such
# that a valid assignment still exists
#
proc fit_to_end_chain { cix } {
	global snbVar snbConst mySeq myChain myConnect mySearch mySpectra

	# exit if the overlapping connection and chain data disagree
	set nConnect $myChain($myChain(current),nConnect)
	set pix [ test_compatible $cix 1 $nConnect 0 ]
	if { $pix == -1 } {
		return
	}
	set myConnect($cix,pix) $pix

	# keep track of best score of the valid possible assignments
	set bestStat $snbConst(MISSING_STAT) 

	# Look through the entire sequence.
	set nTest [expr $mySeq(seqLen) - $nConnect]
	for {set aaIx 1} {$aaIx < $nTest} {incr aaIx} {
		set location [expr $aaIx + $nConnect + 1]

		# Only consider start of possible assignment chains
		if { $mySeq($aaIx,testAssign) == "START_POSSIBLE" } {
			set scor [ fit_test_residue $location $cix $pix 1 ]
			if { $scor < $bestStat } {
				set bestStat $scor
			}
		}
	}

	# add to our search list
	if { $bestStat < $snbVar(bestfit,sdevTerm) } {
		set iProb $mySpectra(connect,PROB)
		set fit [ calc_fit_scor $bestStat ]
		if {[ catch { set prob $myConnect($cix,$pix,$iProb) } err]} {
			set prob 0.0
		}
		lappend mySearch(foundList) "$prob $fit $cix $pix" 
	} 
}

######################################################################
#
# Can this connectivity replace a connectivity in the current chain such
# that a valid assignment still exists
#
proc fit_to_middle_chain { cix } {
	global snbVar snbConst mySeq myChain mySpectra myConnect mySearch

	# exit if the overlapping connection and chain data disagree
	set nConnect $myChain($myChain(current),nConnect)
	set item $snbVar(panel,chainSelect)
	set pix [ test_compatible $cix 0 $item 0 ]
	if { $pix == -1 } { return }
	set pix [ test_compatible $cix 1 $item 1 ]
	if { $pix == -1 } { return }
	set myConnect($cix,pix) $pix

	# Check sequence fitting
	set bestStat $snbConst(MISSING_STAT) 
	set nTest [expr $mySeq(seqLen) - $nConnect]
	for {set aaIx 1} {$aaIx < $nTest} {incr aaIx} {
		# Only consider start of possible assignment chains
		if { $mySeq($aaIx,testAssign) == "START_POSSIBLE" } {
			set location [expr $aaIx + $item - 1]
			set next [expr $aaIx + $item]
			set r0 [ fit_test_residue $location $cix 1 0 ]
			if { $r0 > $bestStat } { continue }
			set r1 [ fit_test_residue $next $cix $pix 1 ]
			if { $r1 > $bestStat } { continue }
			if { $r1 > $r0 } { 
				set bestStat $r1
			} else {
				set bestStat $r0
			}
		}
	}

	set iProb $mySpectra(connect,PROB)
	set fit [ calc_fit_scor $bestStat ]
	if {[ catch { set prob $myConnect($cix,$pix,$iProb) } err]} {
		set prob 0.0
	}
	lappend mySearch(foundList) "$prob $fit $cix $pix" 
}

######################################################################
#
# Can this connectivity be assigned anywhere in the sequence, no current
# chain. 
#
proc fit_to_empty_chain { cix } {
	global snbVar snbConst mySeq mySearch myConnect mySpectra

	set pNum $myConnect($cix,pNum)
	for {set pix 1} {$pix <= $pNum} {incr pix} {
		set bestStat($pix) $snbVar(bestfit,sdevTerm)
	}

	# search for best fit somewhere in the sequence
	for {set aaIx 1} {$aaIx < $mySeq(seqLen)} {incr aaIx} {
		set r0 [ fit_test_residue $aaIx $cix 1 0 ]
		if { $r0 < $snbVar(bestfit,sdevTerm) } {
			set location [expr $aaIx + 1]
			for {set pix 1} {$pix <= $pNum} {incr pix} {
				if { $r0 < $bestStat($pix) } { 
					set r1 [ fit_test_residue $location $cix $pix 1 ]
					if { $r1 < $bestStat($pix) } {
						if { $r1 > $r0 } { 
							set bestStat($pix) $r1
						} else {
							set bestStat($pix) $r0
						}
					}
				}
			}
		}
	}

	# add to our search list if we found a good one
	set iProb $mySpectra(connect,PROB)
	for {set pix 1} {$pix <= $pNum} {incr pix} {
		if { $bestStat($pix) < $snbVar(bestfit,sdevTerm) } {
			set fit [ calc_fit_scor $bestStat($pix) ]
			if {[ catch { set prob $myConnect($cix,$pix,$iProb) } err]} {
				set prob 0.0
			}
			lappend mySearch(foundList) "$prob $fit $cix $pix"
		}
	}
}

######################################################################
#
# Convert a test statistic for closeness of fit to an integer 0-9 
#
proc calc_fit_scor { tStat } {
	global snbVar snbConst

	# incompatible fit
	if { $tStat > $snbConst(MAX_STAT) } { return -1 }

	# calculate integer score based on test stat of shifts fit.
	set iScor 0
	set cutoff $snbVar(bestfit,sdevTerm)
	while { $tStat <= $cutoff && $iScor <= 9 } {
		incr iScor
		set cutoff [expr $cutoff * 0.5]
	}
	return $iScor
}

######################################################################
######################################################################
######################################################################

######################################################################
#
#  find best place to fit shifts in the current chain to the sequence 
#
proc fit_find_best { } {
	global snbVar snbConst myChain mySeq

	set cur $myChain(current)
	set numTests [expr $mySeq(seqLen) - $myChain($cur,nConnect)]
	if { $numTests == $mySeq(seqLen) } { return -1 }

	set bestScor $snbConst(EXTRA_STAT) 
	set bestIx 0

	# test current chain at each place in the sequence
	for {set aaIx 1} {$aaIx <= $numTests} {incr aaIx} {
		set scor [ fit_test_chain $aaIx 0 ]
		if {$scor < $bestScor} { 
			set bestIx $aaIx
			set bestScor $scor
		}
	}
	
	# return index of best fit in the sequence
	return $bestIx
}

######################################################################
#
# test proposed chemical shifts in the current chain to a spot in the sequence  
# Either return the score of the worst fitting shift of all atoms in the
# chain or exit as soon as a shift is found to be above some level
#
proc fit_test_chain { aaIx exitEarly } {
	global snbVar snbConst myChain myConnect

	set cur $myChain(current)
	set numConnect $myChain($cur,nConnect)
	set worst 0.0
	for {set i 1} {$i <= $numConnect} {incr i} {
		set cix $myChain($cur,$i,cix)
		set pix $myChain($cur,$i,pix)

		# find shifts from previous connection
		if { $myConnect($cix,$pix,cStatus) == $snbConst(NO_VAL) && $i > 1 } {
			set j [expr $i - 1]
			set cix $myChain($cur,$j,cix)
			set pix $myChain($cur,$j,pix)
			set scor [ fit_test_residue $aaIx $cix $pix 1 ]
		} else {
			set scor [ fit_test_residue $aaIx $cix $pix 0 ]
		}
		if { $scor > $worst } {
			if {$scor >= $snbVar(bestfit,sdevTerm) && $exitEarly == 1} {
				return $scor
			}
			set worst $scor
		}
		incr aaIx 
	}

	# test last residue in the chain
	set pix $myChain($cur,$numConnect,pix)
	set scor [ fit_test_residue $aaIx $cix $pix 1 ]
	if { $scor > $worst } { return $scor }
	return $worst
}

######################################################################
#
# test proposed chemical shifts of one residue in a chain
# return the score of the worst fitting shift of the atoms in the residue
#
proc fit_test_residue { aaIx cix pix n } {
	global snbVar snbConst myConnect mySeq mySpectra

	set hi 0
	foreach atomName $mySpectra(atoms,names) {
		set atom $mySpectra(connect,$atomName,$n)
		if {[ catch { set shift $myConnect($cix,$pix,$atom) } err]} {
			return 0
		}
		set rc [fit_test_shift $aaIx $shift $atomName]
		if { $rc < $snbConst(MAX_STAT) } {
			if {$rc > $hi} { 
				set hi $rc 
			}
		} else {
			if { $rc == $snbConst(MISSING_STAT) || $rc == $snbConst(EXTRA_STAT) } {
				return $rc
			}
		}
	}
	return $hi 
}

######################################################################
#
# test one proposed chemical shift
#
proc fit_test_shift { aaIx shift atom } {
	global snbVar snbConst mySeq

	if { $shift != $snbConst(NO_VAL) } { 
		if {[ catch { set mean $mySeq($aaIx,$atom,mean) } err]} {
			return $snbConst(EXTRA_STAT) 
		} else {
			return [expr abs(($mean - $shift) / $mySeq($aaIx,$atom,sdev))]
		}
	}
	if {[ catch { set mean $mySeq($aaIx,$atom,mean) } err]} {
		return $snbConst(NA_STAT) 
	} else {
		return $snbConst(MISSING_STAT) 
	}
}

######################################################################
#
# See if a given connection (cix) has a variation (pix) which
# is connectable to the current chain.
#
proc test_compatible { cix connectItem connectNum iEdit } {
	global snbVar snbConst myChain myConnect mySpectra

	# We gather relevant peakids for connection in current chain
	set cur $myChain(current)
	set cixChain $myChain($cur,$connectNum,cix)
	set pixChain $myChain($cur,$connectNum,pix)

	# Cannot do comparision, this connectivity record is not based on real data
	if { $pixChain == $snbConst(NO_VAL) } {
		return $snbConst(NO_VAL)
	}

	# Go through all variations of connections specified by cix
	# Find the variation (if it exists) where the peakids match
	# between the proposed connection and the current chain
	set pNum $myConnect($cix,pNum)
	for {set pix 1} {$pix <= $pNum} {incr pix} {
		set count 0
		foreach i $mySpectra(connect,PKID,$iEdit) {
			set pkId $myConnect($cix,$pix,$i)
			set ix [lindex $mySpectra(connect,PKID,$connectItem) $count]
			if { $pkId != $myConnect($cixChain,$pixChain,$ix) } {
				set count 0
				break
			}
			incr count
		}

		# We found a connection variation with corresponding peakids
		if { $count > 0 } {
			return $pix
		}
	}

	# This proposed connection cannot be added to current chain 
	return -1 
}
