
######################################################################
#
# graphics routines for the main smartnotebook window 
#
######################################################################

######################################################################
#
# start up smartnotebook main window 
#
proc panel_start_snb { } {
	global snbVar snbConst

	set title " smartnotebook - $snbVar(VERSION) - $snbVar(VERSION_DATE) "
	set f [usnb_show_win snbmain $title]
	if { $f == "FALSE" } { return }
	panel_snb_main
}

######################################################################
#
# redraw entire smartnotebook main window 
#
proc panel_restart_snb { } {
	global snbVar snbConst

	usnb_kill_win_save $snbVar(snbmain,winName)
	panel_start_snb
}

######################################################################
#
# main smartnotebook screen layout 
#
proc panel_snb_main { } {
	global snbVar snbConst myChain

	get_snb_settings
	set f $snbVar(snbmain,winName).frame

	pack [frame $f] -expand true -fill both
	create_menubar $f 

	# search section
	create_search $f.pSearch
	pack $f.pSearch -pady 10

	# connection and chaining sections
	set fpan $f.panels
    frame $fpan
	pack $fpan -expand true -fill both
	create_connect $fpan.connect
	create_chain $fpan.chain
	pack $fpan.connect $fpan.chain -side left -anchor ne -expand true -fill both

	# buttons section
	create_buttons $f.buttons
	pack $f.buttons

	# sequence section
	create_sequence $f.seq
	pack $f.seq

	# start in a disabled state
	panel_state_buttons

	# add chains, suggestions and current chain
	panel_menubar_chains
    chains_menu_select $myChain(current)
	panel_menubar_suggest
}

######################################################################
#
# pulldown menus
#
proc create_menubar { f } {
	global snbVar snbConst

	frame $f.menu
	pack $f.menu -fill x 
	set mb $f.menu.menubar
	frame $mb -relief raised -bd 1
	frame $f.menu.dummy
	pack $mb $f.menu -side top -fill x
	menubar_items $mb
}

######################################################################
#
# create menu items
#
proc menubar_items { f } {
	global snbVar snbConst varGui

	menubutton $f.mFile -text 		"File" \
		-menu $f.mFile.menu
	menubutton $f.mOptions -text 	"Options" \
		-menu $f.mOptions.menu
	menubutton $f.mChains -text 	"Chains" \
		-menu $f.mChains.menu
	menubutton $f.mSuggest -text 	"Suggest" \
		-menu $f.mSuggest.menu
	menubutton $f.mHelp -text 		"Help" \
		-menu $f.mHelp.menu

	# File menu
    menu $f.mFile.menu
	$f.mFile.menu add command -label "View Log File...  " \
			-command "u_file_view snbVar(fn,log)"
	$f.mFile.menu add command -label "Import Connectivities File...  " \
			-command "file_import_connect"
	$f.mFile.menu add separator
	$f.mFile.menu add command -label "Quit smartnotebook  " \
		-command "file_snb_exit"
	$f.mFile.menu add separator
	$f.mFile.menu add command -label "Quit nmrview  " \
		-command "file_snb_exit; exit"

	# Options menu
    menu $f.mOptions.menu -tearoffcommand u_tearoff -tearoff 1

    $f.mOptions.menu add command -label "pipp_nav" \
        -command "pipp_nav"

	$f.mOptions.menu add command -label "Configure/Update" \
		-command "options_init"
	$f.mOptions.menu add separator

	$f.mOptions.menu add command -label "Folded Peaks" \
		-command "folded_peaks_main"
	$f.mOptions.menu add command -label "Create Connectivity" \
		-command "submit_add_main"
	$f.mOptions.menu add separator

	# Peak Labels menu
	$f.mOptions.menu add command -label "Spectra: Peak Labels" \
		-command "options_peaklabel"

	# Vertical Scale menu 
	$f.mOptions.menu add command -label "Spectra: Vertical Scale" \
		-command "options_vscale"

	# Preferences menu
	$f.mOptions.menu add separator
	$f.mOptions.menu add command -label "Save Window Geometry" \
		-command "usnb_save_geometry"
	$f.mOptions.menu add command -label "Preferences" \
		-command "options_settings"

	# Help menu
	set varGui(menubarHelp) $f.mHelp.menu
    menu $varGui(menubarHelp)
	$f.mHelp.menu add command -label "About" \
		-command "about_help"
	$f.mHelp.menu add command -label "Latest News  " \
		-command "sys_run_browser $snbVar(help,latest)"
	$f.mHelp.menu add separator
	$f.mHelp.menu add command -label "$snbVar(VERSION) Manual " \
		-command "sys_run_browser $snbVar(help,home)"
	$f.mHelp.menu add command -label "Examples " \
		-command "sys_run_browser $snbVar(help,examples)"

	# Chains menu
	set varGui(menubarChains) $f.mChains.menu
    menu $varGui(menubarChains) -tearoffcommand u_tearoff -tearoff 1

	# Suggest Chains menu
	set varGui(menubarSuggest) $f.mSuggest.menu
    menu $varGui(menubarSuggest) -tearoffcommand u_tearoff -tearoff 1

	pack $f.mFile $f.mOptions $f.mChains $f.mSuggest -side left
	pack $f.mHelp -side right
	tk_menuBar $f $f.mFile $f.mOptions $f.mChains $f.mSuggest $f.mHelp
	focus $f
}

######################################################################
#
# create menu of suggested chains 
#
proc panel_menubar_suggest { } {
	global snbVar snbConst varGui mySuggest

	set num $mySuggest(chainNum)

	$varGui(menubarSuggest) delete 0 end
	$varGui(menubarSuggest) add command -label "Suggest Chains..." \
		-command suggest_panel

	if { $num > 0 } {
		$varGui(menubarSuggest) add command -label "--------------------" 
		for {set j 1} {$j <= $num} {incr j} {
			if { $mySuggest($j,nConnect) > 0 } {
				$varGui(menubarSuggest) add command -label \
					$mySuggest($j,cLabel) -command "suggest_menu_select $j"
			}
		}
	}
}

######################################################################
#
# create menu of chains, the contents of this menu item changes frequently 
#
proc panel_menubar_chains { } {
	global snbVar snbConst varGui myChain

	$varGui(menubarChains) delete 0 end
	$varGui(menubarChains) add command -label "Start new chain" \
		-command chains_menu_new
	$varGui(menubarChains) add command -label "Delete current chain" \
		-command { chains_menu_delete }
	$varGui(menubarChains) add command -label "--------------------" 

	set num $myChain(chainNum)
	for {set j 1} {$j <= $num} {incr j} {
		if { $myChain($j,nConnect) > 0 } {
			$varGui(menubarChains) add command -label $myChain($j,cLabel) \
				-command "chains_menu_select $j"
		}
	}
}

######################################################################
#
# search panel section 
#
proc create_search { f } {
	global snbVar snbConst varGui myConnect mySpectra

	frame $f

	# reference Id section
	set g1 $f.ref
	frame $g1
	label $g1.label -text "Reference Id:" -anchor e
	eval { entry $g1.entry -width 8 -relief sunken -bd 2 } -textvariable snbVar(panel,refId)
	wrap_add_tip $g1.entry g1entry $snbVar(balloon,refid)
	pack $g1.label -side left
	pack $g1.entry -side right -fill x -expand true
	# where reference Id is shown or edited
	bind $g1.entry <Return> search_from_keyboard
	set snbVar(snbmain,searchEntry) $g1.entry

	# Search connection section
	set g2 $f.buttons
	frame $g2
	button $g2.set -text "Spectra" \
		-padx $snbVar(snbwin,padx) \
		-command search_from_spectra
	wrap_add_tip $g2.set g1but $snbVar(balloon,spectra)
	set varGui(search) $g2.searchButton
	button $varGui(search) -text "Search" \
		-padx $snbVar(snbwin,padx) \
		-command search_terms
	wrap_add_tip $varGui(search) g2but $snbVar(balloon,search)
	pack $g2.set $varGui(search) -side left

	if ![info exists mySpectra(nv,file,$mySpectra(hsqc,nv,0))] {
		$g2.set configure -state disabled
	}

	pack $g1 $g2 -padx 2 -side left
	set snbVar(panel,ruleSelect) "all" 
}

######################################################################
#
# Create menu items for search patterns.
#
proc create_search_patterns { rule } {
	global snbVar snbConst varGui

	# set the pattern search rule
#	set snbVar(panel,ruleSelect) $rule
#	$varGui(ruleSelect) configure -text "Pattern: $rule"

	# re-do our previous search with new pattern
#	search_ref_ids 0 0 $snbConst(P_PREV) 0
}

######################################################################
#
# connect panel section 
#
proc create_connect { f } {
	global snbVar snbConst mySearch

	frame $f
    frame $f.pConnect
	pack $f.pConnect -expand true -fill both -padx $snbVar(snbmain,pConnect,padx)
	label $f.pConnect.label -text "    Fit   Connect  Pattern Stat"
	pack $f.pConnect.label -anchor w 

	set mySearch(items) [listbox $f.pConnect.listing \
		-setgrid false -selectmode browse \
		-yscrollcommand "$f.pConnect.scroll set"]
	scrollbar $f.pConnect.scroll -command "$f.pConnect.listing yview"
	pack $f.pConnect.scroll -side right -fill y
	pack $f.pConnect.listing -side left -fill both -expand true
	bind $mySearch(items) <ButtonRelease-1> \
		[list connect_panel_click $mySearch(items) %y]

	set mySearch(searchNum) 0
}

######################################################################
#
# buttons section 
#
proc create_buttons { f } {
	global snbVar snbConst varGui

	pack [frame $f] -pady 5 -padx 5
	frame $f.top
	frame $f.bot
	pack $f.top $f.bot

	frame $f.top.buttons
	frame $f.bot.buttons
	pack $f.top.buttons
	pack $f.bot.buttons

	set varGui(addToChainButton) $f.top.buttons.addButton
	set varGui(deleteButton) $f.top.buttons.deleteButton
	set varGui(drawSpectraButton) $f.bot.buttons.displayButton
	set varGui(showbestButton) $f.bot.buttons.fitButton

	button $varGui(addToChainButton) -text "Add to Chain" \
		-padx $snbVar(snbwin,padx) \
		-width $snbVar(snbmain,command,width) \
		-command "chains_add_button"
	wrap_add_tip $varGui(addToChainButton) dsp $snbVar(balloon,add_chain)
	button $varGui(deleteButton) -text "Delete from Chain" \
		-padx $snbVar(snbwin,padx) \
		-width $snbVar(snbmain,command,width) \
		-command "chains_delete_button"
	wrap_add_tip $varGui(deleteButton) dsp $snbVar(balloon,del_chain)
	button $varGui(drawSpectraButton) -text "Display Spectra" \
		-padx $snbVar(snbwin,padx) \
		-width $snbVar(snbmain,command,width) \
		-command "usnb_update_spectra 1" 
	wrap_add_tip $varGui(drawSpectraButton) dsp $snbVar(balloon,disp_spec)
	button $varGui(showbestButton) -text "Best Fit" \
		-padx $snbVar(snbwin,padx) \
		-width $snbVar(snbmain,command,width) \
		-command "sequence_best_assign"
	wrap_add_tip $varGui(showbestButton) dsp $snbVar(balloon,best_fit)

	bind $varGui(drawSpectraButton) <ButtonPress> \
		"set snbVar(cursor,loc) 0"
	#  With double click, User sees crosshairs on ca1 cb1 instead of ca0 cb0
	bind $varGui(drawSpectraButton) <Double-ButtonPress> \
		"set snbVar(cursor,loc) 1"

	pack $varGui(addToChainButton) $varGui(deleteButton) -side left
	pack $varGui(drawSpectraButton) $varGui(showbestButton) -side left

}

######################################################################
#
# chains panel section 
#
proc create_chain { f } {
	global snbVar snbConst

	frame $f
    frame $f.pChain
	pack $f.pChain -expand true -fill both -padx $snbVar(snbmain,pChain,padx)
	label $f.pChain.label -text "Current Chain"
	pack $f.pChain.label

	set snbVar(panel,chainList) [listbox $f.pChain.listing \
		-setgrid false -selectmode browse \
		-yscrollcommand "$f.pChain.scroll set"]
	scrollbar $f.pChain.scroll -command "$f.pChain.listing yview"
	pack $f.pChain.scroll -side right -fill y
	pack $f.pChain.listing -side left -fill both -expand true
	bind $snbVar(panel,chainList) <ButtonRelease-1> \
		[list chains_panel_click $f.buttons $snbVar(panel,chainList) %y]
}

######################################################################
#
# sequence display panel
#
proc create_sequence { f } {
	global snbVar snbConst mySeq varGui

	frame $f
    frame $f.nseq
	pack $f.nseq -pady 5
	label $f.nseq.label -text "Sequence"
	pack $f.nseq.label -anchor w

	set varGui(seqTable) $f.nseq.nassign
    frame $varGui(seqTable)
	pack $varGui(seqTable)

	set rowNum 0
	set colNum [expr $snbVar(snbmain,colMax) + 1]
	for {set aaIx 1} {$aaIx <= $mySeq(seqLen)} {incr aaIx} {
		if {$colNum > $snbVar(snbmain,colMax)} {
			incr rowNum
			frame $varGui(seqTable).$rowNum
			pack $varGui(seqTable).$rowNum -anchor w
			set numtxt [format "%3d" $mySeq($aaIx,aaNum)]
			label $varGui(seqTable).$rowNum.pRes -text $numtxt -width 3
			pack $varGui(seqTable).$rowNum.pRes -side left
			set colNum 1
		}
		set name1 $mySeq($aaIx,name1) 
		set resBut $varGui(seqTable).$rowNum.$colNum
		label $resBut -text $name1 -relief raised -pady 4 -width 1
		pack $resBut -side left
		bind $resBut <Button-1> "sequence_shifts_fit $aaIx"
		bind $resBut <Enter> "set_seq_current $f.nseq.label $aaIx" 
		bind $resBut <Leave> "$f.nseq.label configure -text Sequence" 
		incr colNum
	}
}

######################################################################
#
# set sequence residue Id in title 
#
proc set_seq_current { f aaIx } {
	global snbVar snbConst mySeq

	set numtxt $mySeq($aaIx,aaNum)
	$f configure -text "Residue Id: $numtxt"
}

######################################################################
#
# find colors for one residue in the sequence 
#
proc panel_residue_color { aaIx } {
	global snbVar snbConst varGui mySeq myChain

	set rowNum [expr ($aaIx - 1) / $snbVar(snbmain,colMax) + 1]
	set colNum [expr ($aaIx - 1) % $snbVar(snbmain,colMax) + 1]
	set cur $myChain(current)

	if { $mySeq($aaIx,assignChainIx) >= 0 } {

		# assigned current chain
		if { $cur == $mySeq($aaIx,assignChainIx) } {
			$varGui(seqTable).$rowNum.$colNum configure \
				-background $snbVar(snbmain,assignCur,bgColor) 
			$varGui(seqTable).$rowNum.$colNum configure \
				-fg $snbVar(snbmain,assignCur,fgColor) 

		# assigned not current chain 
		} else {
			$varGui(seqTable).$rowNum.$colNum configure -background \
				$snbVar(snbmain,assign,bgColor)
			$varGui(seqTable).$rowNum.$colNum configure -fg \
				$snbVar(snbmain,assign,fgColor) 
		}
		return
	}

	# possible assign, start of chain
	if { $mySeq($aaIx,testAssign) == "START_POSSIBLE" } {
		$varGui(seqTable).$rowNum.$colNum configure -background \
			$snbVar(snbmain,pos1Assign,bgColor)
		$varGui(seqTable).$rowNum.$colNum configure -fg \
			$snbVar(snbmain,pos1Assign,fgColor)
		return
	}

	# possible assign
	if { $mySeq($aaIx,testAssign) == "POSSIBLE" } {
		$varGui(seqTable).$rowNum.$colNum configure -background \
			$snbVar(snbmain,posAssign,bgColor) 
		$varGui(seqTable).$rowNum.$colNum configure -fg \
			$snbVar(snbmain,posAssign,fgColor) 
		return
	}

	# none of the above
	$varGui(seqTable).$rowNum.$colNum configure -background \
		$snbVar(snbmain,unAssign,bgColor)
	$varGui(seqTable).$rowNum.$colNum configure -fg \
		$snbVar(snbmain,unAssign,fgColor)
}

######################################################################
#
# set state of connect buttons and menu 
#
proc panel_state_buttons { args } {
	global snbVar snbConst myChain varGui

	$varGui(addToChainButton) configure -state disabled 
	$varGui(drawSpectraButton) configure -state disabled 
	$varGui(deleteButton) configure -state disabled

	if { [string first add $args] >= 0 } {
		$varGui(addToChainButton) configure -state normal
	}
	
	if { [string first draw $args] >= 0 } {
		$varGui(drawSpectraButton) configure -state normal
	}

	if { [string first delete $args] >= 0 } {
		if { $myChain($myChain(current),aaIx) < 0 } {
			$varGui(deleteButton) configure -state normal
		}
	}
}

######################################################################
#
# Let user know the software is busy by changing cursor 
#
proc panel_state_disable { } {
	global snbVar snbConst 

	foreach i $snbConst(WINDOWS) {
		if [winfo exists $snbVar($i,winName)] {
			$snbVar($i,winName) config -cursor watch
		}
	}
	if [winfo exists $snbVar(snbmain,winName)] {
		$snbVar(snbmain,searchEntry) config -cursor watch
		$snbVar(snbmain,winName) config -cursor watch
	}

	update
}

######################################################################
#
# Let user know the software is ready to accept inputs by changing cursor
#
proc panel_state_enable { } {
	global snbVar snbConst 

	foreach i $snbConst(WINDOWS) {
		if [winfo exists $snbVar($i,winName)] {
			$snbVar($i,winName) config -cursor $snbVar(snbwin,defaultCursor) 
		}
	}
	if [winfo exists $snbVar(snbmain,winName)] {
		$snbVar(snbmain,winName) config -cursor $snbVar(snbwin,defaultCursor) 
		$snbVar(snbmain,searchEntry) config -cursor $snbVar(snbwin,insertCursor) 
	}

	update
}

######################################################################
#
# set font for snb
#
proc get_snb_settings { } {
	global snbVar snbConst

	# set font variable
	if { $snbVar(window,fontSize) == "default" } {
		set snbVar(window,font) "$snbVar(window,fontFamily)"
		set snbVar(window,fontbold) "$snbVar(window,fontFamily)bold"
	} else {
		set snbVar(window,font) "$snbVar(window,fontFamily) $snbVar(window,fontSize)"
		set snbVar(window,fontbold) "$snbVar(window,font) bold"
	}
	if { $snbVar(text,fontSize) == "default" } {
		set snbVar(text,font) "$snbVar(text,fontFamily)"
	} else {
		set snbVar(text,font) "$snbVar(text,fontFamily) $snbVar(text,fontSize)"
	}

	# window font items
	foreach i "orb xalign snbmain snbinit snbmsg" {
		option add *$i*Font "$snbVar(window,font)"
	}

	# bold font items
	option add *boldLabel*Font "$snbVar(window,fontbold)"

	# fixed font items
	option add *snbmain*pConnect*Font "$snbVar(text,font)"
	option add *snbmain*pChain*Font "$snbVar(text,font)"
	option add *bestfit*Font "$snbVar(text,font)"
	option add *orb*results*Font "$snbVar(text,font)"
	option add *xalign*results*Font "$snbVar(text,font)"
}

######################################################################
#
# pop up an "About" window
#
proc about_help { } {
	global snbVar

	
	set title " About smartnotebook " 
	set f [usnb_show_win snbabout $title]
	if { $f == "FALSE" } { return }

	# frame to hold logos
    frame $f.frabout1
	pack $f.frabout1


	canvas $f.frabout1.pence1
	set imageFile $snbVar(INSTALL)/lib/docs/images/pence.gif
	image create photo pencephoto -format gif -file $imageFile
	$f.frabout1.pence1 create image 0 0 -anchor nw -image pencephoto
	pack $f.frabout1.pence1 -side left -expand true -fill both
	
	canvas $f.frabout1.pence2
	set imageFile $snbVar(INSTALL)/lib/docs/images/mrc.gif
	image create photo mrcphoto -format gif -file $imageFile
	$f.frabout1.pence2 create image 0 0 -anchor nw -image mrcphoto
	pack $f.frabout1.pence2 -side left -expand true -fill both

	frame $f.text
	pack $f.text
	label $f.text.lbabout1 -text "smartnotebook\ncopyright Sept 2002\nAuthors: Carolyn Slupsky/Robert Boyko (programmer)/Valerie Booth/Brian Sykes\nDepartment of Biochemistry\nUniversity of Alberta\nEdmonton, Alberta, Canada"
	pack $f.text.lbabout1
	
}
