
######################################################################
#
# main smartnotebook tcl program
#
######################################################################

######################################################################
#
# user types "snb" or "smartnotebook" at the nmrview console 
#
proc snb { } {
	snb_start_main
}
proc smartnotebook { } {
	snb_start_main
}

######################################################################
#
# check state of smartnotebook 
#
proc snb_start_main { } {
	global snbVar snbConst

	# snb window already exists!  
	if [info exists snbVar(snbmain,winName)] {
		set f $snbVar(snbmain,winName)
		if [winfo exists $f] {
			usnb_log all "Main smartnotebook window already exists!"
			if { [wm state $f] == "iconic" } { wm deiconify $f }
			raise $f
			return
		}
	}

	define_snb_world
	snb_startup
}

######################################################################
#
# define smartnotebook world
#
proc define_snb_world { } {
	global snbVar snbConst myChain mySeq mySpectra myConnect

	# just making sure the world really is uninitialized 
	foreach aVar "snbVar snbConst mySeq myConnect mySpectra myChain" {
		if [info exists $aVar] {
			unset $aVar
		}
	}

	# make sure I can see where snb is installed.
	define_snb_installation

	# read in all the tcl scripts associated with snb
	read_snb_tcl_scripts

	# read constants and variable settings
	define_snb_constants
	prefs_constants
	define_snb_vars

	usnb_log stdout "Starting Smartnotebook (snb)"
}

######################################################################
#
# check for proper installation of snb 
#
proc define_snb_installation { } {
	global snbVar snbConst

	# check for environment variable SMARTNOTEBOOK 
	if [catch {set snbVar(INSTALL) [exec printenv SMARTNOTEBOOK]} err] {
		usnb_err "I do not know where SMARTNOTEBOOK software is installed.\
			You should make sure your NMRVIEW startup script is setting \
			the SMARTNOTEBOOK environment variable."
	}

	# make sure installation of smartnotebook exists
	if ![file isdirectory $snbVar(INSTALL)] {
		usnb_err "SMARTNOTEBOOK install dir not found: \
			'$snbVar(INSTALL)'"
	}
	if ![file isdirectory $snbVar(INSTALL)/lib] {
		usnb_err "SMARTNOTEBOOK install lib not found: \
			'$snbVar(INSTALL)/lib'"
	}

	# read current version of smartnotebook
	if {[ catch { set fvers [open $snbVar(INSTALL)/lib/VERSION r ] } err]} {
		usnb_err "Cannot read version control file '$snbVar(INSTALL)/lib/VERSION'"
	}
	gets $fvers snbVar(VERSION)
	gets $fvers snbVar(VERSION_DATE)
	close $fvers
}

######################################################################
#
# Read in all the tcl scripts associated with snb 
#
proc read_snb_tcl_scripts { } {
	global snbVar snbConst 

#	read in balloon help variables
	source $snbVar(INSTALL)/lib/help/balloon.txt

	set tcl_path $snbVar(INSTALL)/tcl
	source $tcl_path/assign.tcl
	source $tcl_path/chains.tcl
	source $tcl_path/connect.tcl
	source $tcl_path/data.tcl
	source $tcl_path/file.tcl
	source $tcl_path/fit.tcl
	source $tcl_path/folded.tcl
	source $tcl_path/init.tcl
	source $tcl_path/nvdraw.tcl
	source $tcl_path/options.tcl
	source $tcl_path/orb.tcl
	source $tcl_path/panel.tcl
	source $tcl_path/prefs.tcl
	source $tcl_path/search.tcl
	source $tcl_path/sequence.tcl
	source $tcl_path/snb.tcl
	source $tcl_path/snb_pn.tcl
	source $tcl_path/submit.tcl
	source $tcl_path/suggest.tcl
	source $tcl_path/sys.tcl
	source $tcl_path/usnb.tcl
	source $tcl_path/util.tcl
	source $tcl_path/xalign.tcl
}

######################################################################
#
# define program constants 
#
proc define_snb_constants { } {
	global snbVar snbConst

	# basic constants
	set snbConst(COMMENT)			#
	set snbConst(MAX_DIM)			4
	set snbConst(DIM_LABELS)		{x y z a}
	set snbConst(MAX_CONNECT)		20000

	set snbConst(NO_VAL)			-99
	set snbConst(NO_RULE)			"???"	

	# Names of all the windows
	set snbConst(WINDOWS)			"bestfit search preferences fold1 \
		fold2 fold3 reinit dialog peaklabel vscale submit1 submit2 showfile"

	# codes for shift analysis
	set snbConst(MAX_STAT)			100
	set snbConst(NA_STAT)			1000
	set snbConst(MISSING_STAT)		1001
	set snbConst(EXTRA_STAT)		1002

	# status types for a connection record
	set snbConst(C_OK)				1
	set snbConst(C_DELETE)			2

	# status types for assign records
	set snbConst(A_UNASSIGN)		1
	set snbConst(A_CHAIN)			2
	set snbConst(A_ASSIGN)			3

	# generic error codes
	set snbConst(ERR)				-1
	set snbConst(OK)				0

	# codes to denote where in the chain to apply a given operation
	set snbConst(P_START)			1
	set snbConst(P_MID)				2
	set snbConst(P_END)				3
	set snbConst(P_EMPTY)			4
	# codes for additional chain operation types
	set snbConst(P_PREV)			5
	set snbConst(P_ALL)				6
	set snbConst(P_NEW)				7

	# the name we give to snb configuration files
	set snbConst(F_DEFAULTS)		snb.defaults
	set snbConst(F_INIT)			snb.init

	# variables set in each preferences menu
	set snbConst(P_GEOMETRY)       1
	set snbConst(P_PEAKLABEL)      2
	set snbConst(P_VSCALE)         3
	set snbConst(P_SETTINGS)       4

	# read data file constants 
	set snbConst(E_SEQ)            1
	set snbConst(E_XPK)            2
	set snbConst(E_PEAKCON)        3
	set snbConst(E_CONNECT)        4
	set snbConst(E_CHAINS)         5

	# search type constants 
	set snbConst(S_NONE)           1
	set snbConst(S_CURRENT)        2
	set snbConst(S_NEXT_UNUSED)    3
	set snbConst(S_PREV_UNUSED)    4
	set snbConst(S_NEXT_UNASSIGN)  5
	set snbConst(S_PREV_UNASSIGN)  6
	set snbConst(0,S_TITLE)        "ResId(i-1)" 
	set snbConst(1,S_TITLE)        "ResId(i)"
	set snbConst(AMINO_ACIDS)		[list A-ala C-cys D-asp E-glu F-phe \
									G-gly  H-his I-ile K-lys L-leu M-met \
									N-asn P-pro Q-gln R-arg S-ser T-thr \
									V-val W-trp Y-tyr]

	# current selected connection in chains/connect panels
	set snbConst(panelSelect)      "->"
}

######################################################################
#
# Give default values to snb variables. 
#
proc define_snb_vars { } {
	global snbVar snbConst

	# This is the file prefix which identifies program(s) within
	# snbVar(fn,expdir)which snb will run to create connectivity
	# data files.
	set snbVar(parm,c_prog)       "peakcon"

	# This is the file prefix which identifies the connectivity output files
	# that smartnotebook will read. These files may have been generated by the
	# programs in snbVar(fn,expdir) or created by the user. 
    set snbVar(parm,c_out)        "con"

	# Do we read updated peakpick files for nmrview for every change in
	# assignment? (1=yes, 0=no). The default is no, too slow otherwise.
    set snbVar(parm,readpks)      0

	# The default is that we do not label peaks in the spectra (too slow
	# and can get messy) 
    set snbVar(parm,peaksLabelMode) NO

	# Does the user want snb to calculate default connection records
	# (1=yes, 0=no)
	set snbVar(parm,calcConnect)  1

	# Help files used by snb 
	set root "$snbVar(INSTALL)/lib/docs"
	set snbVar(help,latest)       "http://www.pence.ca/software/smartnotebook/index.html"
	set snbVar(help,intro)        "$root/intro.html"
	set snbVar(help,home)         "$root/index.html"
	set snbVar(help,examples)     "$root/startup.html"
	set snbVar(help,seq)          "$root/io.html"

	# Where is the directory of supported experiments 
    set snbVar(fn,expdir)         "$snbVar(INSTALL)/lib/experiments"

	# Where is the index file which defines all the experiments. 
    set snbVar(fn,expList)        "$snbVar(fn,expdir)/INDEX"

	# This is the default location for the log file.
	set snbVar(fn,log)            "./snb.log"

	# This is the default location for the program verbose debug file.
	set snbVar(fn,debug)          "./.snb.debug"

	# This is the default output directory where snb keeps all its output files.
    set snbVar(fn,session)        "./snb.out"

	# Default location for the input sequence file. 
    set snbVar(fn,seq)            ""
	set fn [glob -nocomplain ./*.seq]
	if { [string length $fn] > 0 } {
		set snbVar(fn,seq) [lindex $fn 0]
	}

	# Default location for the input shift constraints file. 
    set snbVar(fn,shifts)         "$snbVar(fn,session)/shifts"

	# Default location for user's application defaults file. 
	set snbVar(fn,defaults,user)  "$snbVar(INSTALL)/lib/$snbConst(F_DEFAULTS)"

	# Default location for the input ppm.out file. 
    set snbVar(fn,ppm)            ""

	# Default rule type for user submitted connections. 
    set snbVar(parm,urule)        "uxx"

	# User submitted connectivities file
	set snbVar(fn,submit) "$snbVar(fn,session)/$snbVar(parm,c_out).submit.uxx"

	# Default location for the imported connections directory 
    set snbVar(fn,connect,importDir) [sys_get_pwd]

	# crosshair selection 
	set snbVar(cursor,loc) 0

	# window names
	set snbVar(snbmain,winName)     .snbmain
	set snbVar(snbinit,winName)     .snbinit
	set snbVar(snbmsg,winName)      .snbmsg
	foreach i $snbConst(WINDOWS) {
		set snbVar($i,winName)      $snbVar(snbmain,winName).$i
	}
}

######################################################################
######################################################################

######################################################################
#
#  Smartnotebook world is all defined, start it up.
#
proc snb_startup { } {
	global snbVar snbConst mySearch mySpectra mySuggest

	# how is user data configured? 
	configure_snb
	if { $snbVar(OKFLAG) != 0 } {
		usnb_log all "Aborting smartnotebook"
		return
	}

    set mySearch(searchNum) 0
	set mySuggest(chainNum) 0
    set snbVar(panel,chainSelect) 0
    set snbVar(panel,connectSelect) -1

	# window stuff used by tcl, hsqc window for mouse events
	foreach w $mySpectra(winList) {
		set mySpectra($w,winName) ".$w"
	}

	# look for mouse clicks in reference window, may not be needed anymore?
	# set w $mySpectra(exp,ref)
	# set mySpectra(mouselisten) ".$w.0_0"

	usnb_show_msg "Calculating connectivities, please wait..."

	# read data files
	snb_read_data $snbConst(E_SEQ)

	# tell nmrview about assigned peaks
	nvdraw_update_peaklists $snbVar(parm,readpks)
	
	# remind user about folded peaks
	folded_peaks_check

	# show main window and await commands
	usnb_kill_win_nosave $snbVar(snbmsg,winName)
	panel_start_snb

	usnb_log file "" 
	usnb_log file "NOTE: Turn up standard deviation on assignment if expected connections \n appear missing (eg use BestFit button)!" 
	usnb_log stdout "Ready"
}

######################################################################
#
# make sure custom configuration files are properly setup 
#
proc configure_snb { } {
	global snbVar snbConst gui myChain mySearch mySpectra myPeak

	sys_remove_file $snbVar(fn,debug)

	usnb_log file "***************************************************"
	usnb_log all "Smartnotebook session: [sys_get_date]"
	usnb_log file "***************************************************"
	usnb_log file " "
	usnb_log all "Version of smartnotebook: $snbVar(VERSION)"
	usnb_log all "Installation directory: $snbVar(INSTALL)"
	usnb_log all " "

	# Read in the global smartnotebook application defaults file
	set snbVar(fn,defaults,lib)  "$snbVar(INSTALL)/lib/$snbConst(F_DEFAULTS)"
	if ![file readable $snbVar(fn,defaults,lib)] {
		usnb_err "Configuration file not readable: '$snbVar(fn,defaults,lib)'"
	}
	usnb_log all "Sourcing global defaults: '$snbVar(fn,defaults,lib)'"
	source $snbVar(fn,defaults,lib)
	
	# Read the user initialization file (./snb.init) 
	# This will tell snb where all the user snb files are located. 
	set snbVar(OKFLAG) 0
	set snbVar(fn,snbInit,user) "./$snbConst(F_INIT)"
	if ![file readable $snbVar(fn,snbInit,user)] {
		# Exit if we had any problems
		if { [init_snb_win] == 1 } {return}
	}
	usnb_log all "Sourcing user initialization: '$snbVar(fn,snbInit,user)'"
	source $snbVar(fn,snbInit,user) 

	# Test for defined amino acids file for "Search" window 
	if ![info exists mySpectra(fn,searchItems)] {
		set mySpectra(fn,searchItems) " "
	}
	if ![file readable $mySpectra(fn,searchItems)] {
		set mySpectra(fn,searchItems) "$snbVar(fn,expdir)/$mySpectra(exp,type)/$mySpectra(exp,ref).search"
	}

	# Test for rules directory
	if ![info exists mySpectra(fn,rulesdir)] {
		set mySpectra(fn,rulesdir) " "
	}
	if ![file isdirectory $mySpectra(fn,rulesdir)] {
		set mySpectra(fn,rulesdir) "$snbVar(fn,expdir)/$mySpectra(exp,type)/$mySpectra(exp,ref).rules"
	}

	# Test current web browser setting 
	if ![info exists snbVar(fn,browser)] {
		set snbVar(fn,browser) " "
	}
	if ![file readable $snbVar(fn,browser)] {
		usnb_find_browser
	}

	# Read the user app defaults
	if [file readable $snbVar(fn,defaults,user)] {
		usnb_log all "Sourcing user defaults: '$snbVar(fn,defaults,user)'"
		source $snbVar(fn,defaults,user) 
	} else {
		usnb_log all "No user defaults file: '$snbVar(fn,defaults,user)'"
	}

	# Make sure output directory exists
	if ![file isdirectory $snbVar(fn,session)] {
		usnb_err "Output directory does not exist: '$snbVar(fn,session)'.\
		If you don't know where it is, remove ./$snbConst(F_INIT) and restart snb."
	}

	# check output directory for compatibility 
	usnb_check_version

	# Read in the user experiment configuration file
	if ![file readable $mySpectra(fn,expInit,user)] {
		if ![info exists mySpectra(fn,expInit,lib)] {
			set mySpectra(fn,expInit,lib) "$snbVar(fn,expdir)/$mySpectra(exp,type)/$mySpectra(exp,ref)-$mySpectra(exp,spec)"
		}
		if [file readable $mySpectra(fn,expInit,lib)] {
			sys_copy_file $mySpectra(fn,expInit,lib) $mySpectra(fn,expInit,user)
		} else {
			usnb_err "Unable to read config file: '$mySpectra(fn,expInit,lib)'"
		}
	}
	usnb_log all "Sourcing user snb experiment configuration: '$mySpectra(fn,expInit,user)'"
	source $mySpectra(fn,expInit,user)

	# Read in the user experiment nmrview configuration file 
	if ![file readable $mySpectra(fn,nvInit,user)] {
		if ![info exists mySpectra(fn,nvInit,lib)] {
			set mySpectra(fn,nvInit,lib) "$snbVar(fn,expdir)/$mySpectra(exp,type)/$mySpectra(exp,ref)-$mySpectra(exp,spec).nmrview"
		}
		if [file readable $mySpectra(fn,nvInit,lib)] {
			sys_copy_file $mySpectra(fn,nvInit,lib) $mySpectra(fn,nvInit,user)
		} else {
			usnb_err "Nmrview config not readable: '$mySpectra(fn,nvInit,lib)'"
		}
	}
	usnb_log all "Sourcing user nmrview experiment configuration: '$mySpectra(fn,nvInit,user)'"
	usnb_log all "Tell nmrview to draw the spectral windows..."
	usnb_log all " "
	source $mySpectra(fn,nvInit,user)
}

######################################################################
#
# This procedure gets called when smartnotebook starts up or 
# whenever there are changes in the peak lists.
#
proc snb_read_data { entryPoint } {
	global snbVar snbConst myPeak mySuggest mySpectra myChain

	# Start at the beginning, establish sequence data 
	if { $entryPoint <= $snbConst(E_SEQ) } {

		# read in chemical shifts constraints file
		usnb_log all " "
		usnb_log all "Read shift constraints file: '$snbVar(fn,shifts)'."
		data_read_shifts $snbVar(fn,shifts)

		# read previous assignments, if any, in the input ppm.out file
		usnb_log all " "
		usnb_log file "Read assignments made outside snb: '$snbVar(fn,ppm)'."
		data_read_ppm $snbVar(fn,ppm) ppmAssign
	}
	
	# reading in the original peak pick files
	if { $entryPoint <= $snbConst(E_XPK) } {

		usnb_log all " "
		usnb_check_spectra file
		usnb_log all "Reading peakpick files specified above."
		data_read_xpk
	
		# then we read in folded peaks (changes for original peakpick files)
		usnb_log all "Checking for folded peaklists."
		folded_read_data $mySpectra(xpk,foldFnames)

		# these are now the output peaklists in our session directory
		usnb_log all "Writing the updated peaklists."
		data_write_xpk $mySpectra(xpk,outFnames)
	}

	# do we need to run software to update connection files?
	# this can take awhile to do.
	if { $entryPoint <= $snbConst(E_PEAKCON) } {
		if { [usnb_check_connect] > 0 } {
			connect_make_data
		} 
	}

	# update our connection data structures 
	if { $entryPoint <= $snbConst(E_CONNECT) } {
		usnb_log all "Reading the following connectivity files."
		data_read_connect
	}

	# read current chains 
	if { $entryPoint <= $snbConst(E_CHAINS) } {
		data_read_chains $snbVar(fn,chains)
		if [file exists $snbVar(fn,chains)] {
			usnb_log stdout "Reading $myChain(chainNum) current chains from\
				'$snbVar(fn,chains)'"
			usnb_log file "\n*** Start Chains File: '$snbVar(fn,chains)'"
			usnb_backup_file $snbVar(fn,chains) $snbVar(fn,log)
			usnb_log file "*** End Start Chains File: '$snbVar(fn,chains)'\n"
		} else {
			usnb_log all "Start with no current chains in '$snbVar(fn,chains)'"
		}
	}

	if [file exists $snbVar(fn,suggest)] {
		suggest_read_chains $snbVar(fn,suggest)
	}

	# finally update output ppm.out file
	data_write_ppm $snbVar(fn,outppm)
}
