############################################################################
#
# pipp_nav
#
# Purpose:  Given a residue number, to "jump" to that residue
#            and put lines through the appropriate atoms.
#
# A program developed at Laval University
#
# email to:	leigh@rsvs.ulaval.ca
# website:      http://nmr.ulaval.ca/labo/software.html
# Development:  Stephane Gagne
# Programmer:   Leigh Willard
# Thanks to: 	Pascal Mercier, University of Alberta,
#		for his additions, bug fixes, and testing
#
#
############################################################################
#
# To invoke:
#       type pipp_nav OR
#       pipp_nav saved_config_file
#       from within nmrview
#
# To install globally (for all users):
#       copy over this tcl program into
#       /usr/local/packages/nmrview5.0.4/nvtcl (or wherever your
#               installation of nmrview is)
#
#       make sure that the tclIndex file is writable by everyone,
#       OR make sure the file is writable for yourself, and start up
#       nmrview.  Everytime nmrview is run, it updates this file
#       based on an inventory of the nvtcl directory, and it needs
#       to be able to index the new program.
#
############################################################################

global const
set const(version) "4.02, Oct 2005"

# revision notes:
#
# VERSION: 4.02, Oct 2005
# change the way the sequence is read in.
#
# VERSION: 4.01, Oct 2005
# small additions to version 4.00
#
# VERSION: 4.00, July 2005
# this version is developed to work alongside smartnotebook.
#
# VERSION: 3.00, Feb 2004
#  -> on startup, program now checks to see if a variable is already
#     set before it re-initializes it.  Useful if you open pipp_nav
#     multiple times within the same session.
#  -> changed the look and feel of most of the program.
#  -> added options for Noesy spectra - the fox indicating
#     the "residue for Y line" (n value)
#  -> Pascal Mercier sent me a fix to properly show the annotations
#     when an expand or full key is hit
# VERSION: 2.10, Dec 2003
#  -> Pascal Mercier sent me a change for the routine make_color_choices
#     and make_patterns_entry to fix problems with the AQUA version.
# VERSION: 2.00, October 2003
#  -> fixed crosshair colours, which weren't being saved before.
#  -> added the use of Shift+arrow keys
#  -> added use of Shift+return key
#  -> double-check for windows before calling them
#  -> give pop-up wait window
#  -> added documentation to this file.
#  -> changed the width bar - used to be ppms now is %
#
############################################################################
# END OF COMMENTS.  PROGRAM FOLLOWS.
############################################################################







#----------------------------------------------------------------------------
# main program.
# put all constants here.

proc pipp_nav { args } {
global const myvar pn_patlist pn_colors pn_pattern

	set const(geometry_help) "+10+500"
	set const(geometry_wait) "+400+300"
	set const(pn_winnames) [list .pn_main .pn_help]
	set const(clear_color) "white"

	if { ! [info exists myvar(barwidth)] } { set myvar(barwidth) 100  }
	if { ! [info exists myvar(geometry)] } { set myvar(geometry) "+10+100" }
	if { ! [info exists myvar(crosshair1)] } { set myvar(crosshair1) Black }
	if { ! [info exists myvar(crosshair2)] } { set myvar(crosshair2) Black }
	if { ! [info exists myvar(res)] } { set myvar(res) 1 }
	if { ! [info exists myvar(label)] } { set myvar(label) 1 }
	if { ! [info exists myvar(side)] } { set myvar(side) 0 }

#	set myvar(xline) "i.HN"
#	set myvar(yline) "i-1.CA|CB"

	set myvar(history_idx) 0
	set myvar(noesy) 0
	set myvar(loading) 0
	set pn_colors [ list Black Red Cyan Green Blue Yellow White ]
	set pn_patlist(X) \
		[ list "i.HA*" "i.HB*" "i.HG*" "i.HD*" "i.HE*"\
		"i.HN"  "i.H*"  "i-1.H"  "i+1.H"  "i.N" \
		"i|i-1.CA|CB"  "i-1.CA|CB"  "i-1.C"  "i|i-1.CA" \
		"i-1.CA"  "i-1.C*" ]
	set pn_patlist(Y) \
		[ list \
		"i.HN"  "i.H*"  "i-1.H"  "i+1.H"  "n.H*"  "i.N"  \
		"i|i-1.CA|CB"  "i-1.CA|CB"  "i-1.C"  "i|i-1.CA" \
		"i-1.CA"  "i-1.C*" "i-1.H*"]
	set pn_patlist(Z) \
		[ list "(none)"   "i.N"  "i.CA"  "i.CB" \
		"i.CG1" "i.CG2" "i.CD1" "i.CD2" "i.CE1" "i.CE2" ] 
	set pn_patlist(A) \
		[ list "constraints file" "shifts from file" "ppm.out from file"]
	set pn_patlist(B) \
		[ list "from NMRVIEW" "from file"]


	pn_init
	pn_key_bindings
	pn_window 


#	get calling arguments.  we are looking for a possible config file name
	if { [llength $args] == 1 } {
		set loadfile [lindex $args 0]
		puts "pipp_nav: loading configuration saved file $loadfile."
		if { [file exists $loadfile] } { pn_load $loadfile }
	}
	tkwait visibility .pn_main

}

#----------------------------------------------------------------------------
# to display the pn window

proc pn_window {} {
global const myvar Nv_Priv snbVar

	destroy .pn_main
	toplevel .pn_main

	wm title  .pn_main "PIPP_NAV - version $const(version)"

	pack [frame .pn_main.main -relief sunken -bd 2] -padx 2 -pady 2
	set f .pn_main.main

# residue number
	pack [frame $f.res -relief raised -bd 4 ] \
		-pady 2 -padx 2
	pack [label $f.res.lab -text "Residue Number: " ] -side left
   	pack [button $f.res.back -bitmap @$snbVar(INSTALL)/lib/images/left.xbm \
		 -width 20 -command { back_res -1 res; pn_navigate } ] \
		-side left -padx 4
   	pack [button $f.res.down -bitmap @$Nv_Priv(image,down) \
		 -width 20 -command { up_res -1 res; pn_navigate } ] \
		-side left
   	pack [button $f.res.up -bitmap @$Nv_Priv(image,up)  \
		-width 20 -command { up_res 1 res; pn_navigate } ] \
		-side left
   	pack [button $f.res.forward -bitmap @$snbVar(INSTALL)/lib/images/right.xbm \
		 -width 20 -command { back_res 1 res; pn_navigate } ] \
		-side left -padx 4
	wrap_add_tip $f.res.back resback $snbVar(balloon,left)
	wrap_add_tip $f.res.down resdown $snbVar(balloon,down)
	wrap_add_tip $f.res.up resup $snbVar(balloon,up)
	wrap_add_tip $f.res.forward resforward $snbVar(balloon,right)

	pack [entry $f.res.ent -width 4 -textvar myvar(res) ] -side left
	pack [button $f.res.go -width 2 -text "GO" \
		-command  { set myvar(res) [check_endpoints res] ; 
		lappend myvar(history) $myvar(res);
		set myvar(history_idx) [expr [llength $myvar(history)] -1];
		pn_navigate }] -side left -padx 5 
# if I use this, then hitting <Shift-Return> triggers pn_navigate
# twice.  Once for the keys binding, and once because of here.
#	bind $f.res.ent <KeyPress-Return> \
#		 { set myvar(res) [check_endpoints res] ; pn_navigate }
	
	
	pack [frame $f.option ] -pady 2
	pack [checkbutton $f.option.chk2 \
		-text "Config" -variable myvar(config) ] -side left 
	pack [checkbutton $f.option.chk1 \
		-text "Options" -variable myvar(options) ] -side left
	pack [checkbutton $f.option.chk3 \
		-text "Extras" -variable myvar(extras) ] -side left
	pack [button $f.option.exit -text Exit -command \
		{destroy .pn_main; destroy .pn_help } ] -side left -padx 2

	trace variable myvar(options) w trace_options
	trace variable myvar(config) w trace_config
	trace variable myvar(extras) w trace_extras

	$f.option.chk1 deselect
	$f.option.chk2 deselect

	set myvar(history) [list $myvar(res) ]
}


#----------------------------------------------------------------------------
# this section is called when the myvar(config) variable gets changed,
# which happens when the user clicks on the "show configuration" checkbox
proc trace_config { arry idx op } {
global myvar

#	if the user re-sizes the window, then the window manager will
#	not let us size the window to fit our widgets.  This command
#	gives us the permission back to resize the window properly
	wm geometry .pn_main ""

	if {$myvar(config) == 1} { pn_config 
	} else  { destroy .pn_main.config }
}


#----------------------------------------------------------------------------
# this section is called when the myvar(options) variable gets changed,
# which happens when the user clicks on the "show options" checkbox
proc trace_options { arry idx op } {
global myvar

#	if the user re-sizes the window, then the window manager will
#	not let us size the window to fit our widgets.  This command
#	gives us the permission back to resize the window properly
	wm geometry .pn_main ""

	if {$myvar(options) == 1} { show_options 
	} else  { destroy .pn_main.options }
}


#----------------------------------------------------------------------------
# this section is called when the myvar(extras) variable gets changed,
# which happens when the user clicks on the "Extras" checkbox
proc trace_extras { arry idx op } {
global myvar

#	if the user re-sizes the window, then the window manager will
#	not let us size the window to fit our widgets.  This command
#	gives us the permission back to resize the window properly
	wm geometry .pn_main ""

	if {$myvar(extras) == 1} { show_extras 
	} else  { destroy .pn_main.main.extras }
}


#----------------------------------------------------------------------------
# this section is called when the user changes the "SEQUENCE" field
# in the configuration panel.
# arry = pn_pattern
proc trace_seq { arry idx op } {
global myvar pn_pattern snbVar

	if { $myvar(loading) == 1} { return }

	# varname = myvar(seq_type)
	set varname ${arry}($idx)
	upvar $varname var

	if { $var == "from file" } {
		set myvar(seq_fullname) [tk_getOpenFile -initialfile pn.seq]
		set seqsplit [split $myvar(seq_fullname) '/'];
		set myvar(seq_from) [lindex $seqsplit "end"];
		nv_sread seq $myvar(seq_fullname)
	}

	if { $var == "from NMRVIEW" } {
		# do nothing.  already loaded in.	
		set myvar(seq_from) "from NMRVIEW"
	}

}


#----------------------------------------------------------------------------
# this section is called when the user changes the "INPUT" field
# in the configuration panel.
# arry = pn_pattern
# idx = hncacb-TFE0,zinput
proc trace_input { arry idx op } {
global myvar pn_pattern snbVar


	if { $myvar(loading) == 1} { return }

	set varname ${arry}($idx)
	upvar $varname var

	if { $var == "ppm.out from file" } {
		set fname [tk_getOpenFile -initialfile ppm.out -title "INPUT PPM FILE"]
		set ret [regexp {^(.+)\/(.+)$} $fname junk dir base]
		if { $ret == 0 } { return }

		# store the filename 
		set myvar(input_from) $base

		# store the directory name, but just show the file name
		# do this by setting the variable named dir
		set myvar(input_dir) $dir

		# store the file type
#		set myvar(input_type) "PPM"

		read_shifts $base $dir $var
	}
		
	if { $var == "shifts from file" } {
		set fname [tk_getOpenFile -initialfile ppm.out -title "INPUT SHIFTS FILE"]
		set ret [regexp {^(.+)\/(.+)$} $fname junk dir base]
		if { $ret == 0 } { return }
		
		# store the filename 
		set myvar(input_from) $base

		# store the directory name, but just show the file name
		# do this by setting the variable name
		set myvar(input_dir) $dir

		# store the file type
#		set myvar(input_type) "SHIFTS"

		read_shifts $base $dir $var
	}

	if { $var == "constraints file" } {
		set fname $snbVar(fn,shifts)
		# check for the directory name
		set ret [regexp {^(.+)\/(.+)$} $fname junk dir base]
		if { $ret == 0 } { 
			set dir "." 
			set base $fname
		}

		# store the filename 
		set myvar(input_from) $base

		# store the directory name, but just show the file name
		# do this by setting the variable named dir
		set myvar(input_dir) $dir

		# store the file type
#		set myvar(input_type) "SHIFTS"
		read_shifts $base $dir $var
	
	}
		
}


#----------------------------------------------------------------------------
# set the default INPUT field from the constraints file
#
proc set_default_input {}  {
global myvar


	set myvar(input_type) "constraints file"
	trace_input myvar input_type ""
	
}


#----------------------------------------------------------------------------
# read any format input file
proc read_shifts { file dir filetype } {
	
	if {$filetype == "ppm.out from file"} { read_ppm_type $file $dir }
	if {$filetype == "shifts from file"} { read_shift_type $file $dir }
	if {$filetype == "constraints file"} { read_shift_type $file $dir }

}


#----------------------------------------------------------------------------
# read a shifts format file into the shifts global data structure.
#
#     shifts(13.HN) 8.529
#
#
proc read_shift_type { base dir } {
global shifts

	# store this in an array (hash table) as 
	set fname "$dir/$base"
	set fp [open $fname r]

#	store the shifts
	while {[gets $fp line] >= 0} {
		set ret [regexp {[0-9]+:[A-Z]+_([0-9]+):([A-Z]+[0-9]*)[ \t]+([+-]*[0-9]+\.*[0-9]*).*} $line junk resnum atm sft]
		if {$ret == 1} {
			set idx "$resnum.$atm"
			set shifts($idx) $sft
		}
	}

}


#----------------------------------------------------------------------------
# read a ppm format file into the shifts global data structure.
#
# ex. shifts(13.HN) 8.529
#
proc read_ppm_type { base dir } {
global shifts

	# store this in an array (hash table) as 
	set fname "$dir/$base"
	set fp [open $fname r]

	while {[gets $fp line] >= 0} {
		set ret [regexp {[ \t]*([0-9]+\.[A-Z]+[0-9]*)[ \t]+([+-]*[0-9]+\.[0-9]+)} $line junk idx sft]
		if {$ret == 1} {
			set shifts($idx) $sft
		}
	}
}


#----------------------------------------------------------------------------
# this section of code draws the "extras" widgets on the screen
proc show_extras {  } {
global const myvar Nv_Priv snbVar


	set f .pn_main.main.extras
	destroy $f

	pack [frame $f  -bd 2] -padx 5 -pady 5

	# buttons
	pack [frame $f.but] -side bottom -pady 5
	pack [button $f.but.clear -text "Clear Lines" -command pn_clear] -side left
	pack [button $f.but.rstrips -text "Reset Axis" -command reset_axis] -side left
	pack [button $f.but.help -text Help -command pn_help ] -side left -padx 2

	wrap_add_tip $f.but.clear clear $snbVar(balloon,clearlines)
	wrap_add_tip $f.but.rstrips clear $snbVar(balloon,resetwin)
	wrap_add_tip $f.but.help clear $snbVar(balloon,help)

}


#----------------------------------------------------------------------------
# this section of code draws the "options" widgets on the screen
proc show_options {  } {
global const myvar Nv_Priv


	set f .pn_main.options
	destroy $f

	pack [frame $f  -relief raised -bd 2] -padx 5 -pady 5

# NOESY section
	pack [frame $f.nsy -relief sunken -bd 2] -pady 5 -padx 5
	pack [label $f.nsy.lab1 -text "OPTIONAL NOESY SECTION"]
	pack [label $f.nsy.lab2 -text "set the Y-LINES to n.H*, then let n = " ]
	pack [frame $f.nsy.fr]
	pack [radiobutton $f.nsy.fr.rb0 -variable myvar(noesy) -value 0 \
		-text "i (default) " ] -anchor w
	pack [radiobutton $f.nsy.fr.rb1 -variable myvar(noesy) -value 1 \
		-text "i-1 " ] -anchor w
	pack [radiobutton $f.nsy.fr.rb2 -variable myvar(noesy) -value 2 \
		-text "i-3 " ] -anchor w
	pack [frame $f.nsy.fr.fr] -anchor w
	pack [radiobutton $f.nsy.fr.fr.rb3 -variable myvar(noesy) -value 3 \
		-text "(manual): " ] -side left
	set myvar(n) $myvar(res)
    	pack [button $f.nsy.fr.fr.down -bitmap @$Nv_Priv(image,down) \
		 -width 20 -command { up_res -1 n} ] \
		-side left
    	pack [button $f.nsy.fr.fr.up -bitmap @$Nv_Priv(image,up)  \
		-width 20 -command { up_res 1 n} ] \
		-side left
	pack [entry $f.nsy.fr.fr.ent -state disabled -width 4 -textvar myvar(n)] -side left
	trace variable myvar(noesy) w { trace_ent_state .pn_main.options.nsy.fr.fr.ent }

# crosshairs colours
	pack [frame $f.cc -relief sunken -bd 0]  -pady 10 -fill x
	make_color_choices $f.cc.1 myvar(crosshair1) \
		"Crosshair 1 (left)   :"
	make_color_choices $f.cc.2 myvar(crosshair2) \
		"Crosshair 2 (right) :"
	

# show labels switch
	pack [frame $f.sl -relief sunken -bd 2] -pady 10
	pack [frame $f.sl.fr1] -side top
	pack [label $f.sl.fr1.lab -text "Show Labels: " ] -side left
	pack [checkbutton $f.sl.fr1.chk -variable myvar(label) ] -side left
	pack [frame $f.sl.fr2] -side bottom
	pack [radiobutton $f.sl.fr2.rb1 -variable myvar(side) -value 0 \
		-text "Labels Left " ] -side left
	pack [radiobutton $f.sl.fr2.rb2 -variable myvar(side) -value 1 \
		-text "Labels Right " ] -side left
	

# width of bars
	pack [frame $f.mid -relief sunken -bd 2  ]\
		 -pady 5 -pady 5 -fill x -expand true
	pack [label $f.mid.lab -text "bar width :\n(% of window width) "]
	pack [label $f.mid.lab2 -textvariable myvar(barwidth) -width 5 ] \
		-side left
	pack [scale $f.mid.scl -variable myvar(barwidth) -orient horizontal \
		-from 0 -to 100.0 -resolution 1  -showvalue false \
		-bigincrement 1 -length 10 ] \
		-side left -padx 5 -pady 5 -fill x -expand true
}


#----------------------------------------------------------------------------
# as part of the NOESY frame, whenever the "manual" checkbox is
# selected, then permit the user to change the entry field
#
proc trace_ent_state { widget arry idx op } {
global myvar

	if {! [winfo exists $widget] } { return }

	if  { $myvar(noesy) == 3 } {
		$widget configure -state normal
	} else {
		$widget configure -state disabled
	}
}

#----------------------------------------------------------------------------
# this procedure is cosmetic - it colours in the crosshair color
# entry widget text the appropriate color.
proc trace_color { widget wvar  arry idx op } {

	if { [winfo exists $widget] } {
		upvar $wvar wcolor
		$widget config -fg $wcolor
	}
}


#----------------------------------------------------------------------------
# to make a pull-down window listing a bunch of color choices. 
#
proc make_color_choices { cc_frame cc_var cc_text } { 
global myvar pn_colors Nv_Priv const tcl_version


	pack [frame $cc_frame ]  -padx 20
	pack [label $cc_frame.lab -text $cc_text ] -side left
	upvar $cc_var cur_color
	set cmd "pack [entry $cc_frame.ent -width 10 \
		-fg $cur_color -textvar $cc_var] \
		-side left"
	eval $cmd
	bind $cc_frame.ent <KeyPress-Return> { change_cross_color }
	set cmd "trace variable $cc_var w { trace_color $cc_frame.ent $cc_var }"
	eval $cmd

#	changes added by pascal mercier to get the program 
#	to work with aqua
        set macintoshaqua 0
        if {$tcl_version>8.3} {
                if {[tk windowingsystem]=="aqua"} {set macintoshaqua 1}
        }
        
        if {!$macintoshaqua} {
                pack [menubutton $cc_frame.mb -bitmap @$Nv_Priv(image,down) \
                -menu $cc_frame.mb.menu -underline 0 \
                -relief sunken -bd 3 ] -side left
        } else {
                pack [menubutton $cc_frame.mb  -text "" \
                -menu $cc_frame.mb.menu -underline 0]  -side left
        }
        

	menu $cc_frame.mb.menu -tearoff 0

	foreach col $pn_colors {
		$cc_frame.mb.menu add radiobutton \
			-command change_cross_color \
			-foreground $col \
			-label $col -value $col -variable $cc_var
	}

}


#----------------------------------------------------------------------------
# to change the crosshair colours
proc change_cross_color {} {
global myvar

	foreach openwin [nv_win open] {

		$openwin crosscolor 1 $myvar(crosshair1)
		$openwin crosscolor 2 $myvar(crosshair2)
	}
	
}


#----------------------------------------------------------------------------
# to either inc or dec the residue number, checking the end points.

proc up_res { myinc myidx} {
global myvar

	incr myvar($myidx) $myinc
	set idx [check_endpoints $myidx]
	set myvar($myidx) $idx

	lappend myvar(history) $idx
	incr myvar(history_idx)

}


#----------------------------------------------------------------------------
# to either go forward or backware to previous residues 

proc back_res { myinc myidx } {
global myvar

	set size [llength $myvar(history)]
	set current [expr $myvar(history_idx) + $myinc]
	if { $current >= $size } { set current [expr $size - 1] }
	if { $current < 0 } { set current 0 }
	set myvar($myidx) [lindex $myvar(history) $current] 
	set myvar(history_idx) $current

}


#----------------------------------------------------------------------------
# to checking the end points of the residue numbers,
# returning the end point if the user has entered something too big/little

proc check_endpoints { myidx } {
global myvar Nv_Nresidues

	if {$Nv_Nresidues == 0} {
		puts "*** ERROR. No residues to display.  Have you loaded a ppm file?"
		return 0
	}

#	get users first residue number
	if [catch {nv_residue 1 thisres} ] { return }
	set min $thisres(num)

#	get users last residue number
	if [catch {nv_residue $Nv_Nresidues thisres} ] { return }
	set max $thisres(num)

	if { $myvar($myidx) < $min } { return $min }
	if { $myvar($myidx) > $max } { 
		return [expr $max ]
	}
	return $myvar($myidx)
}

#----------------------------------------------------------------------------
# given a lines pattern, return the ppm value of N atom 
# this routine is currently not being used.

proc get_ppmN {pattern} {
global myvar

	set ret [regexp {i([+-]*.*)\.(.+)$} $pattern junk incr atom]
	if {$ret < 1} {
		puts "PIPP_NAV internal error:  get_ppmN"
		return -1
	}

	if { $incr == "" } { set incr 0 }
	set aanum [expr $myvar(res) + $incr]
	set atomnum [nv_atom num $aanum.N]
	if  { $atomnum > 0 } { 
		set ppmN [nv_atom elem ppm $atomnum]
	} else { set ppmN -1 }

	return $ppmN
}


#----------------------------------------------------------------------------
# given a lines pattern, return the ppm values of the appropriate atoms
# this procedure will return a list.   lots of regexping here!
# for example, the pattern might be i.H*, and this would return
# a list of all ppm values for the current residue H* atoms. 
#
# example patterns:  i.H*     i|i-1.CA|CB    i-1.CA|CB
#		   i-1.CO     i|i-1.CA       i-1.CA        i-1.C*
#
# example bn:  .hsqc
#
# Return:  I have added the amino acid name to the return list.
#          this is so that the name can be printed out next to the line.
#          So the return will now look like this (for example):
#          [C 175.945 CA 52.857 CB 19.399]
#
proc get_ppms {pattern bn} {
global myvar pn_pattern Nv_Nresidues

	set ppmlist ""

	set ret  [regexp {^(.+)\.(.+)$} $pattern junk one two ]
	if {$ret < 1} {
		puts "error with pattern: $pattern"
		return -1
	}

	set indexes [split $one | ]
	set atoms [split $two | ]

	foreach idx $indexes {

#		check for the special noesy index (optional noesy section)
		set isnoe [regexp {^n.*$} $idx]
		if {$isnoe == 1} { 
			switch $myvar(noesy) {
				0	{ set resnum $myvar(res) }
				1	{ set resnum [expr $myvar(res) - 1] }
				2	{ set resnum [expr $myvar(res) - 3] }
				3	{ set resnum $myvar(n) }
			}	
#		no noesy.  proceed normally.
		} else {
			regexp {i([+-]*.*)$} $idx junk incr
			if { $incr == "" } { set incr 0 }
			set resnum [expr $myvar(res) + $incr]
		}

#		set input $pn_pattern($bn,input)
#		get_ppm_nmrview $idx $atoms $resnum
#		lappend ppmlist [lookup_ppm $idx $atoms $resnum $bn]
# DEBUG
		set result [lookup_ppm $idx $atoms $resnum $bn]
		set ppmlist [concat $ppmlist [lookup_ppm $idx $atoms $resnum $bn]]
	}


	if { [llength ppmlist] > 0 } { return $ppmlist }
	else { return "" }

}



#----------------------------------------------------------------------------
# get the ppm value of a given atom, reading from our data structures.
# note that if the ppm value = -1000000.0 (signifying not defined), then
# it is ignored.
proc lookup_ppm {idx atoms resnum bn} {
global myvar pn_pattern Nv_Nresidues shifts 


	set ppmlist ""
	set atlist [split $atoms]
	foreach atm $atlist {

#		convert the pattern atom into a regexp pattern
#		ie. H* gets converted to H.*
		regsub -all {\*} $atm {.*} regatm


		
#		ex. key = 13.CA
		foreach { key value } [array get shifts] {
			if { $value == -1000000.0 } { continue }
			set cmd "regexp {\^($resnum)\.($regatm)\$} $key junk rn atom"
			set ret [eval $cmd]
			if { $ret }  {
				lappend ppmlist $rn.$atom [expr double($value)]
			}
		}
	}
	return $ppmlist
}


#----------------------------------------------------------------------------
# this routine is called if the user wants to read the ppm values
# from nmrview, instead of directly from a ppm file.
proc get_ppm_nmrview {idx atoms resnum} {
global myvar pn_pattern Nv_Nresidues


#	get the residue index for resnum 'resnum'
	set residx -1
	for {set i 1} {$i <= $Nv_Nresidues} {incr i} {
		if [catch {nv_residue $i mytest} ] { return }
		puts "mytest(num) = $mytest(num)"
		if { $mytest(num) == $resnum } {
			set residx $i
			break
		}
	}

	if { $residx == -1 } {
		return
	}
		

	foreach atm $atoms {

#		convert the pattern atom into a regexp pattern
#		ie. H* gets converted to H.*
		regsub -all {\*} $atm {.*} regatm
		

		if [catch {nv_residue $residx thisres} ] { return }

puts "this res is [array names thisres]"

#		for each atom in this residue.	
		foreach resatom [array names thisres] {

#			ie. regexp {H.*} resatom
			set cmd "regexp {\^$regatm\$} $resatom"
			set ret [eval $cmd]

# finally.  we have found a matching atom.  get its ppm value now.
			if { $ret } {
				set atomnum [nv_atom num $resnum.$resatom]
				if  { $atomnum > 0 } { 
					set ppm [nv_atom elem ppm $atomnum]
				} else { set ppm -1 }
				if { $ppm > -1000 } {
					scan $ppm "%f" ppm_num
					set rn "$resnum.$resatom"
# debug
# puts "rn = $rn"
					lappend ppmlist $rn $ppm_num
				}
			}
		}
			
	}
# debug
puts "done get_ppm_nmrview"
}


#----------------------------------------------------------------------------
# function to return the basename of a window
# input:  ex. .15NHsqc-TFE.0
# return: ex. 15NHsqc-TFE0
#
proc getbasename { win } {

	if { ! [ regexp {^\.(.+)\.(.+)} $win junk basename tailer] } {
		puts "INTERNAL ERROR in pn_navigate: bad window name $win"
		return ""
	}
	return "$basename$tailer"
}

#----------------------------------------------------------------------------
# to show a waiting dialog box
#
proc show_wait { win } {
global const

	destroy $win
	toplevel $win

	wm geometry $win $const(geometry_wait) 
	wm title $win "WAIT"

	pack [frame $win.fr1 -relief sunken -bd 4 ] -pady 10
	pack [label $win.fr1.lbl1 -text "Please wait.\nDrawing windows." ] -pady 20 -padx 10
	pack [button $win.but -text "OK" -command {destroy .pn_wait} ] -side bottom

	
}

#----------------------------------------------------------------------------
proc debug { msg } {
}

#----------------------------------------------------------------------------
# this routine goes through all open windows, and draws them with
# their annotation lines
proc pn_draw_all {} {
global pn_pattern myvar shifts


#	store the current window name
	set curwin [nv_win act]


#	set atomnum [nv_atom num $myvar(res).N]
#	puts "atomnum = $atomnum"
#	if  { $atomnum > 0 } { 
#		set ppmN [nv_atom elem ppm $atomnum]
#puts "ppmN = $ppmN"
#		if { $ppmN < -1000 } { return }
#	} else { return }


	show_wait .pn_wait
	tkwait visibility .pn_wait
	foreach openwin [nv_win open] { 
	

		set bn [getbasename $openwin]

#		do we drawn on this window?  First make sure
#		that this window is defined for us.
		if {! [info exists pn_pattern($bn,chk) ] } {
			set pn_pattern($bn,chk) 0
		}
		if {! $pn_pattern($bn,chk) } { continue }

#		make this window current
		nv_win act $openwin

#		are the axis defined?  before "Display Spectra" is clicked on,
#		the windows can be blank.  check for this.
		set test [nv_win ppm x]
		if {[lindex $test 0] == [lindex $test 1] } { 
			nv_win full
#			puts "INFO:  $bn axis aren't defined.  cannot draw."
#			continue 
		}
		
#		pan to the correct Z plane on 3D data
		set ndim [nv_dataset ndim [nv_win dataset]]

		if { ($ndim > 2) && (! ( $pn_pattern($bn,zlines) == "(none)"  )) } {

			set ppmZ [get_ppms $pn_pattern($bn,zlines) $bn]
			if { [llength $ppmZ] > 0 }  {
				z [lindex $ppmZ 1]
#				puts "DEBUG.  panning to z $ppmZ"
			} else { 
				puts "cannot get Z value for $bn, skipping"
				continue 
			}
		}


#		get the x and y ppm values, in a list 
		set ppmX [get_ppms $pn_pattern($bn,xlines) $bn]
		set ppmY [get_ppms $pn_pattern($bn,ylines) $bn]

		if {  [llength $ppmX] == 0 || [llength $ppmY] == 0 } { 
			puts "missing values for $bn.  skipping."
			continue 
		}

#		pan to view the peaks, if necessary.	
		do_pan $openwin $ppmX $ppmY

#		set the annotation
		pn_set_lines $openwin $bn $ppmX $ppmY

#		draw the screen
#		puts "DEBUG.  drawing on $bn.  ppmX = $ppmX.  ppmY = $ppmY"
#		puts "length ppmX = [llength $ppmX]  length ppmY = [llength $ppmY]"
		NvSpectrumDraw

	}

	destroy .pn_wait
#	set back the current window
	nv_win act $curwin

}



#----------------------------------------------------------------------------
# to check if the peak that we want is shown in the current screen.
# if not, then we will have to navigate there before drawing.
#
proc do_pan { curwin ppmX ppmY } {

	set xmax [lindex [ nv_win ppm x ] 0]
	set xmin [lindex [ nv_win ppm x ] 1]

	set ymin [lindex [ nv_win ppm y ] 0 ]
	set ymax [lindex [ nv_win ppm y ] 1 ]

#	for now, just check the first X and Y coordinate to see if
#	they are on the screen... maybe later make sure they all are.

	set myX [ lindex $ppmX 1 ]
	set myY [ lindex $ppmY 1 ]

	# pan in x direction
	if { ($myX < $xmin) || ($myX > $xmax) } {
		set diffx [expr $xmax - $xmin]
		set newxmin [ expr $myX - (($xmax - $xmin) / 2) ]
		set newxmax [ expr $myX + (($xmax - $xmin) / 2) ]
		nv_win ppm x $newxmax $newxmin
	}

	# pan in y direction
	if { ($myY < $ymin) || ($myY > $ymax) } {
		set diffy [expr $ymax - $ymin]
		set newymin [ expr $myY - (($ymax - $ymin) / 2) ]
		set newymax [ expr $myY + (($ymax - $ymin) / 2) ]
		nv_win ppm y $newymin $newymax
	}

}


#----------------------------------------------------------------------------
# to reset the axis on the windows to what they were before pipp_nav was called
#
proc reset_axis {} {
global myvar pn_pattern

	set curwin [nv_win act]
	foreach openwin [nv_win open] {
		set bn [getbasename $openwin]

		# only do this for "on" windows
		if { [info exists pn_pattern($bn,chk)] && $pn_pattern($bn,chk) } {
			# we haven't stored the coordinates yet.  continue.
			if {! [info exists myvar($bn,ppmx)] } { continue }

			nv_win act $openwin
			set x1 [expr double([lindex $myvar($bn,ppmx) 0])] 
			set x2 [expr double([lindex $myvar($bn,ppmx) 1])]
			nv_win ppm x $x1 $x2

			set y1 [expr double([lindex $myvar($bn,ppmy) 0]) ]
			set y2 [expr double([lindex $myvar($bn,ppmy) 1]) ]
			nv_win ppm y $y1 $y2
		}
		NvSpectrumDraw
	}
	nv_win act $curwin

}


#----------------------------------------------------------------------------
# to clear any navigation lines on the screen.
#
proc pn_clear {} {
global NvSpectrum

	set curwin [nv_win act]
	foreach openwin [nv_win open] {

#		set/clear the annotation variable
		set NvSpectrum($openwin,anno) "" 
		lappend NvSpectrum($openwin,anno) "color Blue" 

#		make this window current
		nv_win act $openwin

		NvSpectrumDraw
	}
	nv_win act $curwin

}



#----------------------------------------------------------------------------
# to do the navigation.  called when the "GO" button is pressed
#
proc pn_navigate {} {
global NvassignPanel NvSpectrum myvar


	# store the original x and y axis, if this is the first navigation
	set curwin [nv_win act]
	foreach openwin [nv_win open] {
		# set this only once
		set bn [getbasename $openwin]
		if {! [info exists myvar($bn,ppmx)] } {
			nv_win act $openwin
			set ppmx [nv_win ppm x]
			set ppmy [nv_win ppm y]
			# if the window isn't initialized yet, the ppmx has the same 2 values
			if {[lindex $ppmx 0] != [lindex $ppmx 1]} {
				set myvar($bn,ppmx) $ppmx
				set myvar($bn,ppmy) $ppmy
			}
		} 
	}
	nv_win act $curwin

# navigate in the nmrview Assign Panel 
	if { [winfo exists .assignPanel] } {
		set NvassignPanel(curres) $myvar(res) 
		NvUpdateAssignPan .assignPanel set 1
	}

#	draw in all windows
	pn_draw_all
}



#----------------------------------------------------------------------------
# set annotation for "mywin" 
#
proc pn_set_lines { mywin bn ppmX ppmY } {
global pn_pattern NvSpectrum

	set NvSpectrum($mywin,anno) "" 
	lappend NvSpectrum($mywin,anno) "color Blue" 


	if {  [llength ppmX] == 0 || 
		[llength ppmY] == 0 } {
		pn_clear
		return
	}

#	parse the patterns
	foreach "xname ppmx" $ppmX {
		foreach "yname ppmy" $ppmY {
			pn_setanno $mywin $xname $ppmx $yname $ppmy
		}
	}

}



#----------------------------------------------------------------------------
# set 1 part of the annotation of a window. 
#
proc pn_setanno { pn_win xname xval yname yval } {
global const NvSpectrum myvar

	nv_win act $pn_win

#	set limits on where the lines are going to be drawn
	set top [lindex [nv_win ppm y] 0]
	set bottom [lindex [nv_win ppm y] 1]

#	set endpoints of bars, depending on what the user specified
	set left [lindex [nv_win ppm x] 0]
	set right [lindex [nv_win ppm x] 1]

	set widthright [expr ($xval - $right) * $myvar(barwidth) * .01 ] 
	set widthleft [expr ($left - $xval) * $myvar(barwidth) * .01 ] 
	set myleft [expr $xval + $widthleft]
	set myright [expr $xval - $widthright]
	if { $myleft > $left } { set myleft $left }
	if { $myright < $right } { set myright $right }

	set step [expr ($left - $right) / 7 ]

#	set cmd "list { line $xval $top $xval $bottom } \
#		{ line $left $yval $right $yval } "
#	
# print lines.  vertical lines are full-screen, horizontal lines vary.
	lappend NvSpectrum($pn_win,anno)  "line $xval $top $xval $bottom"
	lappend NvSpectrum($pn_win,anno)  "line $myleft $yval $myright $yval"

# 	print an optional label
	set xyname "$xname,$yname"
	if { $myvar(label) } {
	
		if { $myvar(side) == 0 } {
		lappend NvSpectrum($pn_win,anno)  "text [expr $left - ($step/10)] $yval $xyname 0"
		} else {
		lappend NvSpectrum($pn_win,anno)  "text [expr $right + $step]  $yval $xyname 0"
		}
	}


}



#----------------------------------------------------------------------------
# procedure contributed by Pascal Mercier (merci!) so that
# the labels stay on the screen properly when the user
# expands or full out the windiw.
proc update_pipp_nav_annotations_expandorfull {} {
global pn_pattern

	set openwin [nv_win act]
	set bn [getbasename $openwin]

#       do we drawn on this window?  First make sure
#       that this window is defined for us.
	if {! [info exists pn_pattern($bn,chk) ] } {
        	set pn_pattern($bn,chk) 0
	}
                
	if {$pn_pattern($bn,chk) } {
#       	get the x and y ppm values, in a list
        	set ppmX [get_ppms $pn_pattern($bn,xlines) $bn]        
        	set ppmY [get_ppms $pn_pattern($bn,ylines) $bn]

#               set the annotation
                pn_set_lines $openwin $bn $ppmX $ppmY
	}

	NvSpectrumDraw
}

#----------------------------------------------------------------------------
# this procedure is a "kludge" to allow the annotations to be
# drawn after these keys have been hit.  The problem is, the command
# "nv_win [draw|expand|etc} does not re-draw the annotations.
# Howver, the command NvSpectrumDraw does.  I have added the code
# to re-draw the annotations after each default key action. 
proc pn_key_bindings {} {
global myvar
	

#Press  button d on keyboard to draw.
	bind Spectrum <d> {
        	NvSpectrumDraw
	}

#Press button e on keyboard to expand box.
	bind Spectrum <e> {
        	nv_win expand 
		update_pipp_nav_annotations_expandorfull
	}

#Press button f on keyboard to get full view.
	bind Spectrum  <f> {
        	nv_win full 
		update_pipp_nav_annotations_expandorfull
	}
#Press button p on keyboard to get the previous view.
	bind Spectrum <p> {
        	nv_win previous 
		NvSpectrumDraw
	}
#Press right arrow to go up one residue number
	bind all <Alt-Right> {
        	up_res 1 res
		pn_navigate
	}
#Press right arrow to go up one residue number
	bind all <Alt-Left> {
        	up_res -1 res
		pn_navigate
	}
#Press up arrow to go to the first residue
	bind all <Alt-Up> {
        	if [catch {nv_residue 1 thisres} ] { return }
        	set myvar(res) $thisres(num)
		pn_navigate
	}
#Press down arrow to go to the last residue
	bind all <Alt-Down> {
        	if [catch {nv_residue $Nv_Nresidues thisres} ] { return }
        	set myvar(res) $thisres(num)
		pn_navigate
	}
#Press enter key to do the same as the GO button 
	bind all <Alt-Return> {
		pn_navigate
	}

}


#----------------------------------------------------------------------------
# initialize the pn_pattern array for each open window
proc pn_init {} {
global pn_pattern myvar


#	get users first residue number
	if [catch {nv_residue 1 thisres} ] { return }
	set myvar(res) $thisres(num)

# 	get all of the open window names	
	foreach openwin [nv_win open] {

		set ret [ regexp {^\.(.+)\.(.+)} $openwin junk basename tailer]
		if { ! $ret } {
			puts "INTERNAL ERROR in pn_navigate: bad window name $openwin"
			return
		}

		set bn $basename$tailer
		if {! [ info exists pn_pattern($bn,chk)] } {
			set pn_pattern($bn,chk) 0
		}
		if {! [ info exists pn_pattern($bn,xlines)] } {
			set pn_pattern($bn,xlines) "" 
		}
		if {! [ info exists pn_pattern($bn,ylines)] } {
			set pn_pattern($bn,ylines) ""
		}
	}
	
}



#----------------------------------------------------------------------------
# this routine brings up another window, allowing the user to enter
# the patterns, and then to possibly save or load those patterns
# from file. 
proc pn_config {} {
global pn_pattern pn_patlist myvar

	
	destroy .pn_main.config
	pack [frame .pn_main.config -relief raised -bd 2 ] -padx 10 -pady 10
	pack [frame .pn_main.config.fr1 ]
	pack [frame .pn_main.config.fr2 ]
	set f .pn_main.config.fr1
	set f2 .pn_main.config.fr2

# 	column which gives all of the open window names	
	pack [frame $f.fr1 ] -pady 10 -side left
	pack [label $f.fr1.lbl -text "ON/OFF" ] -anchor w -padx 5
	foreach openwin [nv_win open] {

		set ret [ regexp {^\.(.+)\.(.+)} $openwin junk basename tailer]
		if { ! $ret } {
			puts "INTERNAL ERROR in pn_navigate: bad window name $openwin"
			return
		}
		set bn $basename$tailer
		pack [checkbutton $f.fr1.$bn -text $openwin \
			-variable pn_pattern($bn,chk) ] -anchor w
	}

# columns containing the X-LINES & Y-LINES & Z-LINES
	pack [frame $f.fr2 ] -pady 10 -side left
	pack [label $f.fr2.lbl -text "X-LINES" ] -anchor w -padx 5
	pack [frame $f.fr3 ] -pady 10 -side left
	pack [label $f.fr3.lbl -text "Y-LINES" ] -anchor w -padx 5
	pack [frame $f.fr4 ] -pady 10 -side left
	pack [label $f.fr4.lbl -text "Z-LINES" ] -anchor w -padx 5


	# INPUT area
	pack [frame $f2.fr1 ] -pady 10
	pack [label $f2.fr1.lbl -text "INPUT" ] -anchor w -padx 5 -side left
	make_patterns_entry $f2.fr1.entZ myvar(input_from) myvar(input_type) \
		$pn_patlist(A) 20
	set test [$f2.fr1.entZ.ent get]
		if { $test == "" } {
			$f2.fr1.entZ.ent insert 0 ""
		}
	trace vdelete myvar(input_type) w {trace_input}
	set cmd "trace variable myvar(input_type) w {trace_input} "
	eval $cmd

	# SEQUENCE area
	pack [frame $f2.fr2 ] -pady 10 -side bottom
	pack [label $f2.fr2.lb1 -text "SEQUENCE" ] -anchor w -padx 5 -side left
	make_patterns_entry $f2.fr2.entZ myvar(seq_from) myvar(seq_type) \
		$pn_patlist(B) 30
	set test [$f2.fr2.entZ.ent get]
		if { $test == "" } {
			$f2.fr2.entZ.ent insert 0 ""
		}
	set cmd "trace variable myvar(seq_type) w {trace_seq} "
	eval $cmd

#	pack [entry $f2.fr2.ent1 -width 30 -textvar myvar(seq_name) ] -side left
#	pack [button $f2.fr2.bt1 -width 6 -text "BROWSE" \
#		-command { set myvar(seq_fullname) \
#		[tk_getOpenFile -initialfile pn.seq] ; 
#		set seqsplit [split $myvar(seq_fullname) '/'];
#		set myvar(seq_name) [lindex $seqsplit "end"];
#		nv_sread seq $myvar(seq_fullname) }
#	] -side left -padx 5



	foreach openwin [nv_win open] {
		set ret [ regexp {^\.(.+)\.(.+)} $openwin junk basename tailer]
		if { ! $ret } {
			puts "INTERNAL ERROR in pn_navigate: bad window name $openwin"
			return
		}
		set bn $basename$tailer
		make_patterns_entry $f.fr2.${bn}entX pn_pattern($bn,xlines) \
			pn_pattern($bn,xlines) $pn_patlist(X) 10
		make_patterns_entry $f.fr3.${bn}entY pn_pattern($bn,ylines) \
			pn_pattern($bn,ylines) $pn_patlist(Y) 10
		make_patterns_entry $f.fr4.${bn}entZ pn_pattern($bn,zlines) \
			pn_pattern($bn,zlines) $pn_patlist(Z) 10
	}


# buttons
	pack [frame .pn_main.config.but] -pady 10  
	pack [button .pn_main.config.but.save -text {Save to File} -command pn_filesave] -side left
	pack [button .pn_main.config.but.load -text {Load from File} -command pn_load] -side left

	
}


#----------------------------------------------------------------------------
# to save the configuration options (the patterns) to a file
#
proc pn_filesave {} {
global pn_pattern const myvar

#	get filename to save to
	set fname [tk_getSaveFile -initialfile pn.config]
	if { $fname == "" } {return}
	set fp [open $fname w]

# 	print comments to the open file
	puts $fp "# "
	puts $fp "# pipp_nav: saved configuration file"
	puts $fp "# "

#	save the window geometry of all open windows.
#	some people might find this very useful
	foreach pnwin $const(pn_winnames) {
		if { [winfo exists $pnwin] } {
			set geom [wm geometry $pnwin]
#			strip off size, just leave x and y
			regexp {\+[0-9]+\+[0-9]+$} $geom geomxy
			puts $fp "if { \[winfo exists $pnwin\] } { wm geometry $pnwin $geomxy }"
		}
	}

#	save the bar width
	puts $fp "set myvar(barwidth) $myvar(barwidth)"

#	save the crosshairs colours
	puts $fp "set myvar(crosshair1) $myvar(crosshair1)"
	puts $fp "set myvar(crosshair2) $myvar(crosshair2)"

#	save the label flag
	puts $fp "set myvar(label) $myvar(label)"
	puts $fp "set myvar(side) $myvar(side)"

#	save the input information
	puts $fp "set myvar(input_from) $myvar(input_from)"
	puts $fp "set myvar(input_dir) $myvar(input_dir)"
	puts $fp "set myvar(input_type) \"$myvar(input_type)\""

#	save the sequence filename (short and long)
	puts $fp "set myvar(seq_from) $myvar(seq_from)"
	puts $fp "set myvar(seq_type) \"$myvar(seq_type)\""
	if { [info exists myvar(seq_fullname)] } {
		puts $fp "set myvar(seq_fullname) $myvar(seq_fullname)"
	}

# 	get all of the open window names	
	foreach openwin [nv_win open] {

		set ret [ regexp {^\.(.+)\.(.+)} $openwin junk basename tailer]
		if { ! $ret } {
			puts "INTERNAL ERROR in pn_navigate: bad window name $openwin"
			return
		}

		set bn $basename$tailer
		puts $fp "set pn_pattern($bn,chk) $pn_pattern($bn,chk)"
		puts $fp "set pn_pattern($bn,xlines) \"$pn_pattern($bn,xlines)\""
		puts $fp "set pn_pattern($bn,ylines) \"$pn_pattern($bn,ylines)\""
		puts $fp "set pn_pattern($bn,zlines) \"$pn_pattern($bn,zlines)\""
	}

	close $fp
	
}


#----------------------------------------------------------------------------
# to load the configuration options (the patterns) to a file
#
proc pn_load {{loadfile ""}} {
global myvar pn_pattern

	set myvar(loading) 1

#	get filename to read from
	if {$loadfile == ""} {
		set loadfile [tk_getOpenFile -initialfile pn.config]
	}

	if { [file exists $loadfile] } {
		source $loadfile
	} else	{
		puts "ERROR: file doesn't exist: $loadfile" 
		return
	}

	if { [info exists myvar(seq_fullname)] } {
		nv_sread seq $myvar(seq_fullname)
	}

	change_cross_color
	set myvar(loading) 0
	if { [info exists myvar(input_from)] && \
		[info exists myvar(input_dir)] && \
		[info exists myvar(input_type)] } {
		read_shifts $myvar(input_from) $myvar(input_dir) $myvar(input_type)
	} else {
		set_default_input
	}
}


#----------------------------------------------------------------------------
# to print out a help message, in the config window 
#
proc pn_help {} {

	set state [winfo exists .pn_help]
	if {$state == 1} {
		destroy .pn_help
	} else {
		pn_help2
	}
}

proc pn_help2 {} {
global const

	
	set f .pn_help
	toplevel $f

	wm geometry $f $const(geometry_help) 
	wm title $f "PIPP_NAV HELP"

	pack [frame $f.fr1]
        pack [text $f.fr1.tx -height 20 -width 80 -yscrollcommand { .pn_help.fr1.sy set} ] -side left
	pack [scrollbar $f.fr1.sy -command {.pn_help.fr1.tx yview}] -side right -expand true -fill y
        pack [button $f.b1 -text Close -default active -command {destroy .pn_help ; .pn_main.main.option.help configure -state normal; return}]

        $f.fr1.tx insert end {  

CONTENTS: 

1) Authors
2) Running the program
3) Using the program
4) Options screen
5) Configuration screen
6) Configuration & patterns
7) Troubleshooting


--------------------------------------------------------------------------
1) AUTHORS:
Leigh Willard (programmer) leigh@rsvs.ulaval.ca
and Stephane Gagne, Laval University, Quebec, Canada.
And many thanks to Pascal Mercier, University of Alberta,
for his additions, bug fixes, and testing.

http://nmr.ulaval.ca/labo/software.html

--------------------------------------------------------------------------
2) RUNNING THE PROGRAM:

The program can be invoked from within nmrview 2 different ways:
1) pipp_nav
2) pipp_nav saved_config_filename
The second way automatically loads a previous saved configuration file.
Otherwise, you may manually load a saved configuration file from within 
the Config panel.

**NOTE** that you need to "configure" the program before you can navigate.

--------------------------------------------------------------------------
3) USING THE PROGRAM

-> To navigate, you may use these short-cut keys:
   Shift+Return  = GO (redraw)
   Shift+right_arrow = go to next residue
   Shift+left_arrow = go to previous residue
   Shift+up_arrow = go to the first residue 
   Shift+down_arrow = go to the last residue 

-> You must "configure" the program before you will be able to navigate.

-> expand previous full 
   In order to use the nmrview functions "expand" "previous" and "full",
   and maintain the lines, I had to re-map the keyboard shortcuts.
   But I did not re-map the menus.  For example, if you type "e" to
   expand, the pipp_nav lines will be maintained, but if you use the 
   menu item "expand" (by right clicking in the spectral window, then
   selecting "View" then "expand") , the lines will be lost.  


--------------------------------------------------------------------------
4) OPTIONS SCREEN

-> The NOESY section will be used whenever the user selects
   the pattern "n.H*" in the Y-LINES column of the config screen
 
-> For the crosshair colors, you may type in a color or use the
   arrows to choose from a list of colors. ( It would be better to set the 
   color of the bars that this program draws, but I have not been able to 
   do that.  If anyone knows how to set colors in annotations, email me. )

--------------------------------------------------------------------------
4) CONFIGURATION SCREEN

You must configure the program before you can navigate.

This screen has 4 columns:

ON/OFF:  a button used to show which window the user wants lines
         drawn in.  If the button is highlighted (coloured in), then
         lines will be drawn in that window.
X-LINES and Y-LINES:   patterns telling the program where to draw the 
         X and Y lines.  The patterns are described below.  
         The user can pick a pre-existing pattern by clicking on the 
         down arrow, or type in their own pattern.
INPUT:   click on the arrow to choose which type of file you want 
         to use.  If you use "constraints", the program automatically
		 obtains the name of the constraints file that you set up
		 with snb.

NOTE:  your patterns are used as soon as you type them in.
       You do not need to "Save to File" for your changes to take effect.
       Conversely, as soon as you "Load from File" your new
       values go into effect.

SAVING:  as well as saving your patterns, this program saves the
       location of your current windows so that the next time you
       load your saved configuration, your windows will reposition. 

--------------------------------------------------------------------------
6) CONFIGURATION & PATTERNS

At the heart of the program lies the ability to read in patterns
and generate appropriate lines for them.  Hopefully the pattern
language is intuitive. 

The simple form is:

   RESIDUE_INDICE.ATOMNAME

To specify more than one RESIDUE_INDICE or ATOMNAME use this notation:

   RESIDUE_INDICE|RESDUE_INDICE.ATOMNAME|ATOMNAME

NOTE:  no spaces are allowed in the pattern.
NOTE:  the bar "|" seperates items.  You may use it as many times as
       you like.

RESIDUE_INDICE = a variable of the form:
	n (used in conjuncation with the noesy section)
	i
	i+1
	i-1
	etc.

ATOMNAME = an atom name, of which * can be used as a wildcard character.
           For example:
           CA
           NH
           C* (means CA and CB and CO)

--------------------------------------------------------------------------
7) TROUBLESHOOTING

These are troubles that we have encountered and their solutions:

problem:
  in the pipp_nav screen, the residue number is not initially set by the
  program (the program should set this initially to the first residue
  number), and everytime the user tried to navigate nothing happened
  and the residue number went blank.
solution:  
  a ppm.out file had not been loaded.  Once it was loaded, 
  everything functioned.


problem:
   the nmrview program was dumping with SEGMENTATION VIOLATION
   when we navigated to certain planes
solution:
   this is an nmrview bug that limits the label size.  If you surpass
   the label size, nmrview will SEGMENT fault when it tries to draw.
   either limit your label sizes, or have nmrview print out the
   residue number instead of the label.



--------------------------------------------------------------------------
}

}
	


#----------------------------------------------------------------------------
# to ask the user where they want the input taken from.
proc input_entry { pe_frame pe_var mypatlist } {

	input_entry $f.fr4.${bn}entZ \
		pn_pattern($bn.input) $pn_patlist(Z)

    pack [frame $pe_frame ]
    set cmd "pack [entry $pe_frame.ent -width 10 -textvar $pe_var] -side left"
    eval $cmd

    if {!$macintoshaqua} {
        pack [menubutton $pe_frame.mb -bitmap @$Nv_Priv(image,down) \
            -menu $pe_frame.mb.menu -underline 0 \
            -relief sunken -bd 3 ] -side left
    } else {
        pack [menubutton $pe_frame.mb -text "" \
            -menu $pe_frame.mb.menu -underline 0 ] -side left
    }

	menu $pe_frame.mb.menu -tearoff 0

	foreach pat $mypatlist {
		$pe_frame.mb.menu add radiobutton \
			-label $pat -value $pat -variable $pe_var
	}


}


#----------------------------------------------------------------------------
# to make a pull-down window listing a bunch of pattern choices. 
#
proc make_patterns_entry { pe_frame pe_text pe_var mypatlist width } { 
global const Nv_Priv pn_pattern tcl_version

        set macintoshaqua 0
        if {$tcl_version>8.3} {
                if {[tk windowingsystem]=="aqua"} {set macintoshaqua 1}
        }
        
        pack [frame $pe_frame ]
        set cmd "pack [entry $pe_frame.ent -width $width -textvar $pe_text] -side left"
        eval $cmd

        if {!$macintoshaqua} {
                pack [menubutton $pe_frame.mb -bitmap @$Nv_Priv(image,down) \
                        -menu $pe_frame.mb.menu -underline 0 \
                        -relief sunken -bd 3 ] -side left
        } else {
                pack [menubutton $pe_frame.mb -text "" \
                        -menu $pe_frame.mb.menu -underline 0 ] -side left
        }

	menu $pe_frame.mb.menu -tearoff 0

	foreach pat $mypatlist {
		$pe_frame.mb.menu add radiobutton \
			-label $pat -value $pat -variable $pe_var
	}


}
