
######################################################################
#
# operating system specific commands 
# this file developed for unix 
#
######################################################################

######################################################################
#
# get current directory 
#
proc sys_get_pwd { } {
	set unixcmd "pwd"
	if [ catch { eval exec $unixcmd } err] {
		usnb_err "Problems executing unix command: '$unixcmd'."
	}
}

######################################################################
#
# get user home directory
#
proc sys_get_home { } {
	set unixcmd "printenv HOME"
	if [ catch { eval exec $unixcmd } err] {
		usnb_err "Problems executing unix command: '$unixcmd'."
	}
}

######################################################################
#
# mkdir command 
#
proc sys_make_dir { dir } {
	set unixcmd "mkdir $dir"
	if [ catch { eval exec $unixcmd } err] {
		usnb_err "Problems executing unix command: $unixcmd \n Check your file permissions."
	}
}

######################################################################
#
# cp command
#
proc sys_copy_file { from to } {
	set unixcmd "cp -r $from $to"
	if [ catch { eval exec $unixcmd } err] {
		usnb_err "Problems with copy command: $unixcmd \n Check your file permissions."
	}
}

######################################################################
#
# create empty file 
#
proc sys_empty_connect { filename } {
	set msg "No dgg connections, sequence has no gly/gly residues."
	if [ catch { eval exec echo #test > $filename } err] {
		usnb_err "Problems with empty connect command\n Check your file permissions."
	}
}

######################################################################
#
# mv command
#
proc sys_move_file { from to } {
	set unixcmd "mv -f $from $to"
	if [ catch { eval exec $unixcmd } err] {
		usnb_err "Problems with move command: $unixcmd \n Check your file permissions."
	}
}

######################################################################
#
# remove file command
#
proc sys_remove_file { fn } {
	if [file isdirectory $fn] {
		set unixcmd "rm -fr $fn"
	} else {
		set unixcmd "rm -f $fn"
	}
	if [ catch { eval exec $unixcmd } err] {
		usnb_err "Problems with remove command: $unixcmd \n Check your file permissions."
	}
}

######################################################################
#
# date command for logging
#
proc sys_get_date { } {
	set unixcmd "date"
	if [ catch { set aTime [eval exec $unixcmd] } err] {
		usnb_err "Problems with date command: $unixcmd \n"
	}
	return $aTime
}

######################################################################
#
# Run program to generate shifts file
#
proc sys_run_makeshift { fseq fout } {
	global snbVar snbConst 

	usnb_log file "Call program to generate chemical shift bounds"
	set cmd "$snbVar(INSTALL)/bin/makeshift $snbVar(INSTALL)/lib/makeshift.def $snbVar(INSTALL)/lib/makeshift.bmrb $fseq > $fout"
	if ![file executable $snbVar(INSTALL)/bin/makeshift] {
		usnb_err "Cannot execute program '$snbVar(INSTALL)/bin/makeshift'"
	}
	if [catch {eval exec $cmd } err] {
		if [file readable $fout] {
			sys_move_file $fout $fout.err
			usnb_err "Problems running $cmd. Please see $fout.err for messages."
		} else {
			usnb_err "makeshift command failed on '$cmd'."
		}
	}
}

######################################################################
#
# Run program to generate chains output file
#
proc sys_run_connect { runScript inputDir outputDir xpkFiles } {
	global snbVar snbConst 

	set outPref "$snbVar(parm,c_out).$snbVar(parm,c_prog)"
	set cmd "$runScript $snbVar(INSTALL)/bin $inputDir $outputDir $outPref $xpkFiles >> $snbVar(fn,debug)"
	if ![file executable $runScript] {
		usnb_err "Cannot execute program '$prog' for creating connections."
	}

	if [catch {eval exec $cmd } err] {
		usnb_err "Problems running $runScript.\n\n Check your xpk files."
	}
}

######################################################################
#
# Run program to show help files
#
proc sys_run_browser { file } {
	global snbVar snbConst 

	set cmd "$snbVar(fn,browser) $file &"
	if [catch {eval exec $cmd } err] {
		usnb_err "Problems running web browser $snbVar(fn,browser) at file:$file.\n"
	}
}
