
######################################################################
#
# simple snb utility scripts
#
######################################################################

######################################################################
#
# look for a web browser 
#
proc usnb_find_browser { } {
	global snbVar snbConst 

	set home [sys_get_home] 

	foreach browser "mozilla netscape Safari"  {
		lappend list "/usr/local/bin/$browser"
		lappend list "/usr/bin/$browser"
		lappend list "/usr/bin/X11/$browser"
		lappend list "/usr/bin/X11/$browser"
		lappend list "$home/bin/$browser"
		lappend list "/Applications/$browser.app/Contents/MacOS/$browser"
		lappend list "$home/Applications/$browser.app/Contents/MacOS/$browser"
	}

	set snbVar(fn,browser) "unknown"
	foreach fn $list {
		if [file executable $fn] {
			set snbVar(fn,browser) $fn
			return
		}
	}
}

######################################################################
#
# ask user a question dialog
#
proc usnb_show_prompt {msg} {
	global snbVar snbConst prompt

#	usnb_debug "Question: $msg"

	# check state of popup window
	set f [usnb_show_win dialog "Dialog Window"]
	if { $f == "FALSE" } {
		usnb_log file "Program check, dialog window was not created."
	}

	frame $f.win
	frame $f.buttons
	pack $f.win -fill both -expand true -pady 10
	pack $f.buttons

	text $f.win.t -setgrid false -wrap word -width 60 -height 5 -relief flat
	$f.win.t insert end $msg
	pack $f.win.t -padx 20 -pady 20 -fill both -expand true

	button $f.buttons.yes -text "YES" \
		-padx $snbVar(snbwin,padx) \
		-command {set prompt(ok) 1}
	button $f.buttons.no -text "NO" \
		-padx $snbVar(snbwin,padx) \
		-command {set prompt(ok) 0}
	pack $f.buttons.yes $f.buttons.no -side left

	tkwait variable prompt(ok)
	usnb_kill_win_save $f

	if {$prompt(ok)} {
#		usnb_debug "Answer: YES"
		return 0
	}
#	usnb_debug "Answer: NO"
	return 1
}

######################################################################
#
# pop a temporary message for the user 
#
proc usnb_show_msg { msg } {
	global snbVar snbConst

	set f [usnb_show_win snbmsg "Smartnotebook Notice"]

	# window exists, just change the message in it
	if { $f == "FALSE" } {
		$snbVar(snbmsg,winName).label configure -text $msg
		return
	}

	label $snbVar(snbmsg,winName).label -text $msg 
	pack $snbVar(snbmsg,winName).label -padx 10 -pady 20
	update
}

######################################################################
#
# raise existing window on desktop or create it if it does not exist
#
proc usnb_show_win { winStr title } {
	global snbVar snbConst

	# new window with "snb" prefix is a toplevel window 
	if { 0 == [string compare -len 3 "snb" $winStr] } {
		set f .$winStr 
	} else {
		# new window should be child of main snb window or init window
		if [winfo exists $snbVar(snbmain,winName)] {
			set f $snbVar(snbmain,winName).$winStr
		}
		if [winfo exists $snbVar(snbinit,winName)] {
			set f $snbVar(snbinit,winName).$winStr
		}
	}

	# if window already exists, make sure user can see it and return
	if [winfo exists $f] {
		set wState [wm state $f]
		if { $wState == "iconic" } {
			wm deiconify $f
		}
		raise $f
		return "FALSE"
	}

	# window does not exist, we should create it and check geometry
	toplevel $f
	wm title $f $title
	if [info exists snbVar($winStr,geometry)] {
		wm geometry $f $snbVar($winStr,geometry)
	}
	return $f 
}

######################################################################
#
# shutdown all windows started by snb
#
proc usnb_destroy_windows { } {
	global snbVar snbConst mySpectra

	# save current geometry
	usnb_save_geometry

	# kill main snb window (which kills all the children too) 
	if [winfo exists $snbVar(snbmain,winName)] {
		destroy $snbVar(snbmain,winName)
	}
	if [winfo exists $snbVar(snbinit,winName)] {
		destroy $snbVar(snbinit,winName)
	}
	if [winfo exists $snbVar(snbmsg,winName)] {
		destroy $snbVar(snbmsg,winName)
	}

	# kill nmrview windows 
	if [info exists mySpectra(winList)] {
		foreach w $mySpectra(winList) {
			if [info exists mySpectra($w,winName)] {
				destroy $mySpectra($w,winName)
			}
		}
	}
}

######################################################################
#
# kill a window, save the geometry of current windows 
#
proc usnb_kill_win_save { f } {
	global snbVar snbConst

	if ![winfo exists $f] {
		return
	}

	usnb_save_geometry
	destroy $f
}	

######################################################################
#
# kill a window, do not save current geometry
#
proc usnb_kill_win_nosave { f } {
	global snbVar snbConst

	if ![winfo exists $f] {
		return
	}
	destroy $f
}	

######################################################################
#
# save geometry of all snb windows 
#
proc usnb_save_geometry { } {
	global snbVar snbConst mySpectra

	# get geometry of all the snb windows
	foreach w "snbmain snbinit snbmsg $snbConst(WINDOWS)" {
		if [info exists snbVar($w,winName)] {
			if [winfo exists $snbVar($w,winName)] {
				set snbVar($w,geometry) [wm geometry $snbVar($w,winName)]
			}
		}
	}

	# get geometry of all the spectral windows
	if [info exists mySpectra(winList)] {
		foreach w $mySpectra(winList) {
			if [info exists mySpectra($w,winName)] {
				if [winfo exists $mySpectra($w,winName)] {
					set mySpectra($w,geometry) [wm geometry $mySpectra($w,winName)]
				}
			}
		}
	}

	# save changes
	prefs_save_parms $snbConst(P_GEOMETRY)
	usnb_log stdout "Saving current snb window geometry."
}

######################################################################
#
# write msg to the log file
#
proc usnb_log { myCode msg } {
	global snbVar snbConst

	# write message to the screen
	if { $myCode == "stdout" || $myCode == "all" } { 
		puts stdout "snb: $msg"
	}

	# put message in log file
	if { $myCode == "file" || $myCode == "all" } { 
		if [catch {open $snbVar(fn,log) a} fileId] {
			puts stdout "Cannot open log file '$snbVar(fn,log)'."
			return
		}
		puts $fileId $msg
		close $fileId
	}
}

######################################################################
#
# append specified file to the log file
#
proc usnb_backup_file { fn dest } {
	global snbVar snbConst

	if [catch {open $dest a} outfp] {
		puts stdout "Cannot open output file for backup: '$dest'."
		return
	}
	if [catch {open $fn r} infp] {
		puts stdout "Cannot open input file for backup: '$fn'."
		return
	}

	while {[gets $infp line] >= 0} {
		puts $outfp $line 
	}

	close $infp
	close $outfp
}

######################################################################
#
# write msg to the debug file
#
proc usnb_debug { msg } {
	global snbVar snbConst 

	# put message in log file
	if [catch {open $snbVar(fn,debug) a} fileId] {
		if [catch {open $snbVar(fn,debug) w} fileId] {
			return
		}
	}
	puts $fileId "snb debug: $msg"
	close $fileId
}

######################################################################
#
# write err to the log file, show in tcl window
#
proc usnb_err { msg } {
	global snbVar snbConst

	bell
	panel_state_enable

	# This is because Macosx tcl error doesn't do what one would think here 
	if [winfo exists $snbVar(snbmain,winName)] { 
		usnb_log all "Error: $msg"
	} else {	
		usnb_log file "Error: $msg"
	}
	error "\n$msg"
}

######################################################################
######################################################################
######################################################################

######################################################################
#   
# Generic update spectral display, call appropriate nmrview routines 
#
proc usnb_update_spectra { flag } {
	global snbVar snbConst myChain mySearch myConnect
	    
	# check for connection to draw selected from chains section
	if {![ catch { set cix $myChain($myChain(current),$snbVar(panel,chainSelect),cix) } err]} {
		if { $myConnect($cix,pNum) == 0 } {
			set pix $snbConst(NO_VAL)
		} else {
			set pix $myChain($myChain(current),$snbVar(panel,chainSelect),pix)
		}
	}

	# check for connection to draw selected from connect panel
	if {![ catch { set cix $mySearch($snbVar(panel,connectSelect),cix) } err]} {
		if { $myConnect($cix,pNum) == 0 } {
			set pix $snbConst(NO_VAL)
		} else {
			set pix $mySearch($snbVar(panel,connectSelect),pix)
		}
	}

	# if no connection, do we draw current panel refId or print an error?
	if ![info exists pix] {
		if { $flag > 0 } {
			usnb_err "Please click on the connection you want to display."
		} else {
			set xy [ usnb_find_ref_values 1 $snbVar(panel,refId) ]
			if { $xy != -1 } {
				nvdraw_ref $xy
			}
		}
		return
	}

	# connection does not have all shift info, draw reference ids instead
	if { $myConnect($cix,rule) == $snbConst(NO_RULE) } {
		set aStr "$myConnect($cix,ref,0)-$myConnect($cix,ref,1)"
		set xy [ usnb_find_ref_values 1 $aStr ]
		if { $xy != -1 } {
			nvdraw_ref $xy
		}
		return
	}

	# draw normal connectivity record
	nvdraw_connect $cix $pix
}

######################################################################
#
# parse a string for reference values 
#
proc usnb_find_ref_values { flag aStr } {
	global snbVar snbConst mySpectra myPeak

	set xpkix $mySpectra(REF,xpkIx)

	# see if user has entered multiple refIds (parse them if yes)
	set connector [ string first - $aStr ]
	if { $connector >= 0 } {
		set ref1 [ string range $aStr 0 [expr $connector - 1] ]
		set ref2 [ string range $aStr [expr $connector + 1] end ]
		if {[ catch { set iPeak $myPeak($xpkix,index,$ref1) } err]} {
			if { $flag == 1 } { return -1 }
			usnb_err "Reference Id entry should be valid refId or 'refId-refId'"
		}
		set peakVal $myPeak($xpkix,origPeak,$iPeak)
		set x1 [lindex $peakVal $mySpectra(REF,xdim)]
		set y1 [lindex $peakVal $mySpectra(REF,ydim)]
	} else {
		set ref2 $aStr
		set x1 $snbConst(NO_VAL)
		set y1 $snbConst(NO_VAL)
	}

	# set x and y values for ref 2.
	if {[ catch { set iPeak $myPeak($xpkix,index,$ref2) } err]} {
		if { $flag == 1 } { return -1 }
		usnb_err "Reference Id entry should be valid refId or 'refId-refId'"
	}
	set peakVal $myPeak($xpkix,origPeak,$iPeak)
	set x2 [lindex $peakVal $mySpectra(REF,xdim)]
	set y2 [lindex $peakVal $mySpectra(REF,ydim)]

	return "$x1 $y1 $x2 $y2"
} 

######################################################################
#
# given a chain record input line, find the corresponding connection record
# it refers to.
#
proc usnb_get_pix { query cix } {
	global snbVar snbConst mySpectra myConnect

	for {set pix 1} {$pix <= $myConnect($cix,pNum)} {incr pix} {
		set flag 1
		foreach resInc "0 1" {
			foreach field $mySpectra(connect,PKID,$resInc) {
				if { [lindex $query $field] != $myConnect($cix,$pix,$field) } {
					set flag 0 
				}
			}
		}
		if { $flag == 1 } {
			return $pix
		}
	}
	return $snbConst(NO_VAL)
}

######################################################################
#
# get peak info from a peaklist data line
#
proc usnb_peak_get { line fieldNums } {
	global snbVar snbConst

	set peak ""
	foreach j $fieldNums {
		set aVal [lindex $line [expr $j-1]]
		lappend peak $aVal
	}
	return $peak
}

######################################################################
#
# check the xpk and nv files. 
#
proc usnb_check_spectra { name } {
	global snbVar snbConst mySpectra

	set mySpectra(xpk,rootNames) ""
	set mySpectra(xpk,outFnames) ""
	set mySpectra(xpk,foldFnames) ""
	usnb_log all "Checking your input peaklists and processed spectral files..."

	# Define output files from original peak pick files 
	for {set i 0} {$i < $mySpectra(xpk,fileNum)} {incr i} {
		set fn $mySpectra(xpk,$name,$i)
		if ![file readable $fn] {
			usnb_err "Peaklist file $mySpectra(xpk,desc,$i) is not readable: '$fn'"
		}
		set root [u_file_prefix $fn "."]
		if { [lsearch $mySpectra(xpk,rootNames) $root] < 0 } {
			lappend mySpectra(xpk,rootNames) $root 
			lappend mySpectra(xpk,outFnames) $snbVar(fn,session)/$root.xpk 
			lappend mySpectra(xpk,foldFnames) $snbVar(fn,session)/$root.fold 
		} else {
			usnb_err "\nProblems with peakpick file name convention: '$root'."
		}
	}

	# Make sure we can see the nmrview spectral files and xpk files
	foreach w $mySpectra(winList) {
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			if ![info exists mySpectra($w,nv,$i)] {
				continue
			}
			if ![info exists mySpectra(nv,$name,$mySpectra($w,nv,$i))] {
				continue
			}
			set nv $mySpectra(nv,$name,$mySpectra($w,nv,$i))
			if ![file readable $nv] {
				set desc $mySpectra(nv,desc,$mySpectra($w,nv,$i))
				# optional spectral file 
				if [string match *optional* $desc] {
					continue
				}
				usnb_err "Processed spectral file not readable: '$nv'"
			}
			if { $mySpectra($w,xpk,$i) < 0 } {
				usnb_log all "  $w spectra: found '$nv', no associated xpk file"
				continue
			}
			# we expect to find an xpk file
			set xpk $mySpectra(xpk,$name,$mySpectra($w,xpk,$i))
			if { [usnb_peaklist_format $xpk label ] == 0 } {
				if { [usnb_peaklist_format $xpk VARS ] == 0 } {
					usnb_err "Peaklist file format unrecognized: '$xpk'"
				}
				set tab [lindex $mySpectra(xpk,rootNames) $mySpectra($w,xpk,$i)]
				if ![file exists $tab.xpk] {
					usnb_log all "Converting tab formatted file '$xpk'"
					usnb_tab2xpk $nv $xpk $tab.xpk
					set mySpectra(xpk,$name,$mySpectra($w,xpk,$i)) $tab.xpk
				}
			} else {
				usnb_log all "  $w spectra: found '$xpk' and '$nv'"
			}
		}
	}
}

######################################################################
#
# check and get par files into current working directory. 
#
proc usnb_check_par { name } {
	global snbVar snbConst mySpectra

	for {set i 0} {$i < $mySpectra(nv,fileNum)} {incr i} {
		set ifn [string trim $mySpectra(par,$name,$i)]
		if { [string length $ifn] > 0 } {
			if ![file readable $ifn] {
				usnb_log "Note: Par file $mySpectra(par,desc,$i) is not readable: '$ifn'"
				continue
			}
			set p [string trim $mySpectra(nv,$name,$i)]
			set root [u_file_prefix $p "."]
			set pfn "./$root.par"
			if ![file readable $pfn] {
				sys_copy_file $ifn $pfn
				usnb_log all "Copying par file '$ifn' to '$pfn'"
			}
		}
	}
}

######################################################################
#
# check if peaklist file has KEY which identifies its type 
#
proc usnb_peaklist_format { fn str } {
	global snbVar snbConst

	set found 0
	set inputfile [open $fn r]
	while {!$found && [gets $inputfile line]!=-1} {
		set found [string match $str* $line]
	}
	return $found
}

######################################################################
#
# check the version of snb we are running, is it compatible with the
# files in the output directory 
#
proc usnb_check_version { } {
	global snbVar snbConst

	# version does not exist, write it
	if ![file exists $snbVar(fn,version)] {
		if {[ catch { set fvers [open $snbVar(fn,version) w ] } err]} {
			usnb_err "Could not open '$snbVar(fn,version)' for writing"
		}
		puts $fvers "$snbVar(VERSION)"
		puts $fvers "$snbVar(VERSION_DATE)"
		close $fvers
		return
	} 

	# read current version
	if {[ catch { set fp [open $snbVar(fn,version) r ] } err]} {
		usnb_err "Cannot read version file '$snbVar(fn,version)'"
	}
	gets $fp snbVar(session_version)
	close $fp

	# if the first 3 characters are the same, then this session directory
	# is compatible with this version of smartnotebook.
	set v1 [string range $snbVar(VERSION) 1 3]
	set v2 [string range $snbVar(session_version) 1 3]
	if { [string compare $v1 $v2] != 0 } {
		set v1 [string range $snbVar(VERSION) 1 1]
		set v2 [string range $snbVar(session_version) 1 1]
		if { [string compare $v1 $v2] == 0 } {
			usnb_err "\nThis version of smartnotebook is $snbVar(VERSION)\
			and this directory was created using $snbVar(session_version).\n\
			You can likely update this snb session by \n\n \
			 mv snb.out/snb.defaults snb.out/snb.defaults.old \n \
			 mv snb.init snb.init.old \n \
			 cp $snbVar(INSTALL)/lib/VERSION snb.out/VERSION\n"
		} else {
			usnb_err "\nThis version of smartnotebook is $snbVar(VERSION)\
			and this directory was created using $snbVar(session_version).\n\
			Please start a fresh directory with no snb output files.\n"
		}
	}
}

######################################################################
#
# check the status of the connection files 
#
proc usnb_check_connect { } {
	global snbVar snbConst mySpectra

	# connect files do not exist and need to be created. 
    set outStr "$snbVar(fn,session)/$snbVar(parm,c_out).$snbVar(parm,c_prog)*"
	if { [ catch { set outFiles [glob $outStr] } err]} {
		return 1
	}
	
	set testFile [lindex $outFiles 0]

	# original peakpick files have been modified, connections need updating
	for {set i 0} {$i < $mySpectra(xpk,fileNum)} {incr i} {
		set origFile $mySpectra(xpk,file,$i)
		if [u_file_newer $origFile $testFile] {
			return 2
		}
	}

	# folded peaks files have been modified, connections need updating
	foreach fn $mySpectra(xpk,foldFnames) {
		if [file exists $fn] {
			if [u_file_newer $fn $testFile] {
				return 3
			}
		}
	}

	# connect files exist and are up to date
	return 0
}

######################################################################
#
# procedure to convert an NMRDraw peak list to NMRView format
#
proc usnb_tab2xpk {dataset nameinputfile nameoutputfile} {

# version 1.1
# 
# written by Pascal Mercier
# pascal.mercier@ualberta.ca
# Ph.D biochemistry
# Canadian Institutes of Health Research (CIHR) 
# Group in Protein Structure and Function
# Department of Biochemistry
# University of Alberta
# Edmonton, AB
# Canada
# Sept 10, 2003

# usage:
# load the dataset in NMRView
# source this tcl script in the NMRView console
# type tabtoxpk name_of_the_dataset name_of_the_NMRDraw_peak_list name_of_converted_xpk_file
# THE EXTENSION XPK IS AUTOMATICALLY ADDED TO THE CONVERTED PEAK LIST
#
# Thanks to Dr. Norma Pawley (LANL) for testing

	global snbVar snbConst

	# get info about the dataset
	nv_dataset open $dataset
	set ndim [nv_dataset ndim $dataset]
	for {set i 1} {$i<=$ndim} {incr i} {
		set label($i) [nv_dataset label $dataset $i]
		set sf($i) [nv_dataset sf $dataset $i]
		set sw($i) [nv_dataset sw $dataset $i]
		set size($i) [nv_dataset size $dataset $i]
	}

	set inputfile  [open $nameinputfile r]
	# read the input file, and find the VARS line

	set foundVARS 0
	while {!$foundVARS && [gets $inputfile line]!=-1} {
		set foundVARS [string match VARS* $line]	
	}
	if {!$foundVARS} {
		usnb_err "No VARS identifier found in '$inputfile', tab2xpk aborting..."
	}

	# write NMRView peak list header
	set outputfile [open $nameoutputfile w]
	puts $outputfile "label dataset sw sf"
	set namedims ""
	set sws ""
	set sfs ""
	set fieldid ""
	for {set i 1} {$i<=$ndim} {incr i} {
		lappend namedims $label($i)
		set sws [concat $sws "{$sw($i)}"]
		set sfs [concat $sfs "{$sf($i)}"]
		set fieldid [concat $fieldid $label($i).L $label($i).P $label($i).W $label($i).B $label($i).E $label($i).J $label($i).U]
	}
	puts $outputfile $namedims
	puts $outputfile $dataset
	puts $outputfile $sws
	puts $outputfile $sfs
	puts $outputfile "$fieldid vol int stat comment flag0"
	
	
	set columnnames [lrange $line 1 end]  
	
	foreach param "INDEX VOL HEIGHT" {
		set column($param) [lsearch $columnnames $param]
		#puts "column $param set to $column($param)"
	}
	
	set dims "X Y Z A"
	#note: the \ trick in the foreach command below does not work if the caracter following the \ is a number
	#must be a tcl thing.  Therefore, put 1 in 3 in variables
	set extra1 1
	set extra3 3
	for {set i 1} {$i<=$ndim} {incr i} {
		set currentdim [lindex $dims [expr $i-1]]
		foreach param "$currentdim\_PPM $currentdim\W_HZ $currentdim$extra1 $currentdim$extra3" {
			set column($param) [lsearch $columnnames $param]
			#puts "column $param set to $column($param)"
		}
	}
	
	# begin conversion
	while {[gets $inputfile line] >= 0} {

		# skip any line whose first column is not a number
		if {[regexp {[0-9]} [lindex $line 0]] && ![regexp {[A-Z]} [lindex $line 0]]} {
			set linetooutput "[expr [lindex $line $column(INDEX)] -1]"
			
		
			for {set i 1} {$i<=$ndim} {incr i} {
				set currentdim [lindex $dims [expr $i-1]]
				set linetooutput [concat $linetooutput "{?}" [lindex $line $column($currentdim\_PPM)]]
				
				# calculate the width of the peak for the appropriate dimension
				set width($i) [format %.3f [expr [lindex $line $column($currentdim\W_HZ)]/$sf($i)]]
				set bound($i) [format %.3f [expr ([lindex $line $column($currentdim$extra3)]-[lindex $line $column($currentdim$extra1)])*$sw($i)/($size($i)*$sf($i))]]
				set linetooutput [concat $linetooutput  $width($i) $bound($i) "++ 0.000 {?}"]
			}
			
			set linetooutput [concat $linetooutput  [lindex $line $column(VOL)] [lindex $line $column(HEIGHT)] "0 {?} 0"]
			puts $outputfile $linetooutput
		}
		
	}
	
	close $inputfile
	close $outputfile
}
