
######################################################################
#
# simple utility scripts
#
######################################################################



######################################################################
#
# to implement tearoff windows in macOSX 
#
proc u_tearoff { w menu } {
    set x [expr {[winfo pointerx $menu] - [winfo width $menu]/2}]
	# Subtract 8 so the mouse is over the titlebar
	set y [expr {[winfo pointery $menu] - 8}]
	wm geometry $menu +$x+$y
	focus -force $menu
}

######################################################################
#
# to make an entry box with slider
#
proc u_slider_entry { f txtvar thistext lo hi res len} {
	pack [frame $f] -anchor w -pady 4
	label $f.label -text $thistext -width 16 
	pack $f.label -side left -anchor s
	scale $f.scale -variable $txtvar -orient horizontal \
		-from $lo -to $hi -resolution $res -length $len
	pack $f.scale -side right -padx 5
}

######################################################################
#
# to make an text box entry
#
proc u_text_entry { f txtvar thistext width } {
	pack [frame $f] -anchor w -pady 4
	pack [label $f.lab -text $thistext] -side left
	set cmd "pack [entry $f.ent -width $width -textvariable $txtvar] -side right"
    eval $cmd
}

######################################################################
#
# to make a label and entry box with selection by pulldown menu
#
proc u_list_entry { myList f tVar txt } {
	frame $f
	pack $f -fill both -expand true
	frame $f.a
	frame $f.b
	pack $f.a $f.b -side left
	label $f.a.label -text $txt -anchor w -width 24
	pack $f.a.label -padx 5 -fill x -expand true
	set cmd "menubutton $f.b.mb -text $tVar \
		-menu $f.b.mb.menu -underline 0 -width 12 \
		-relief sunken -textvariable $tVar"
	eval $cmd
	pack $f.b.mb -side left
	menu $f.b.mb.menu -tearoff 0
	foreach col $myList {
		set cmd "$f.b.mb.menu add radiobutton \
			-value $col -variable $tVar -label $col"
		eval $cmd
	}
}

######################################################################
#
# generic file entry widget
#
proc u_file_entry { f title tWidth tVar eWidth padx cmd browse view } {
    pack [frame $f] -fill x -expand true -padx 1
    pack [frame $f.a] -side left
    pack [frame $f.b] -side left -fill x -expand true
    pack [frame $f.c] -side left
	label $f.a.label -text $title -anchor e -width $tWidth
	pack $f.a.label -side right
	entry $f.b.entry -bd 2 -textvariable $tVar -width $eWidth -relief sunken
#	$f.b.entry config -cursor {xterm white}
	bind $f.b.entry <Return> $cmd
	pack $f.b.entry -side left -fill x -expand true
	if { $browse > 0 } {
		if { $browse == 1 } {
			button $f.c.browse -text Browse \
				-padx $padx \
				-command "u_file_browse $tVar" 
		} else {
			button $f.c.browse -text Browse \
				-padx $padx \
				-command "u_dir_browse $tVar" 
		}
		pack $f.c.browse -side left -padx 8
	}
	if { $view > 0 } {
		button $f.c.view -text View \
			-padx $padx \
			-command "u_file_view $tVar" 
		pack $f.c.view -side left
	}
}

######################################################################
#
# generic command field entry
#
proc u_command_entry {name label width command args} {
	set f $name.frame
    pack [frame $f] -fill x -expand true -padx 1
	label $f.label -text $label -width $width -anchor e
	eval { entry $f.entry -relief sunken -bd 2 } $args
	pack $f.label -side left
	pack $f.entry -side right -fill x -expand true
	bind $f.entry <Return> $command
	return $f.entry
}

######################################################################
#
# purpose: to call a tk routine to search for a file 
#
proc u_file_browse { fn } {
	global snbVar snbConst mySpectra xalignVar orbVar
	set chosen [tk_getOpenFile]
	if { $chosen != "" } { 
		set currentpwd [pwd] 
		set cmd "regsub {$currentpwd\/} $chosen \"\" shortchosen"
		eval $cmd
		set $fn $shortchosen
	}
}

######################################################################
#
# purpose: to call a tk routine to search for a directory 
#
proc u_dir_browse { dir } {
	global snbVar snbConst mySpectra xalignVar orbVar
	set chosen [tk_chooseDirectory]
	if { $chosen != "" } { 
		set currentpwd [pwd]
		set cmd "regsub {$currentpwd\/} $chosen \"\" shortchosen"
		eval $cmd
		set $dir $shortchosen
	}
}

######################################################################
#
# show file view window
#
proc u_file_view { fVar } {
	global snbVar snbConst

	# get file name from variable
	upvar $fVar fn 

	# check state of popup window
	set f [usnb_show_win showfile "File: '$fn'"]
	if { $f == "FALSE" } { return }

	frame $f.win
	frame $f.buttons
	pack $f.win -fill both -expand true
	pack $f.buttons

	# show text window and load file
	u_textwin $f.win.panel 80 30
	u_load_file $fn $f.win.panel.textwin

	# buttons for this window
	button $f.buttons.quitButton \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $f"
	pack $f.buttons.quitButton -side left 
}

######################################################################
#   
# create a generic widget for file text display
#       
proc u_textwin { f width height } {
	frame $f
	pack $f -expand true -fill both -padx 10 -pady 10
    text $f.textwin -relief raised -bd 2 \
		-width $width -height $height \
		-setgrid false -wrap none \
		-xscrollcommand [list $f.xscroll set] \
		-yscrollcommand [list $f.yscroll set]
	scrollbar $f.xscroll -orient horizontal \
		-command [list $f.textwin xview]
	scrollbar $f.yscroll -orient vertical \
		-command [list $f.textwin yview]
	pack $f.xscroll -side bottom -fill x
	pack $f.yscroll -side right -fill y
	pack $f.textwin -side left -fill both -expand true
}

######################################################################
#   
# loads a file onto a canvas - code taken form "TCL and the TK toolkit"
#       
proc u_load_file { file textItem } {
    $textItem delete 1.0 end
    if [catch {open $file r} f] {
		$textItem insert end "Not able to read selected file."
		return -1
	}
	while {![eof $f]} {
		$textItem insert end [read $f 10000]
	}
	close $f
}   

######################################################################
#
# raise existing window on desktop or create it if it does not exist
#
proc u_pop_win { f title } {
	# if window exists, make sure user can see it and return
	if [winfo exists $f] {
		set wState [wm state $f]
		if { $wState == "iconic" } {
			wm deiconify $f
		}
		raise $f
		return "FALSE"
	}

	# window does not exist, we should create it
	toplevel $f
	wm title $f $title
	return TRUE 
}

######################################################################
######################################################################

######################################################################
#
# set a variable if it does not already exist
#
proc u_set_val {var val} {
	upvar $var a
	set val [string trim $val]
	if ![info exists a] {
		set a $val
	}
}

######################################################################
#
# convert amino acid 3 letter code to 1 letter code
#
proc u_resname_3to1 { name3 } {
	set upper [string toupper $name3]
	switch -regexp $upper {
		ASP { return D }
		GLU { return E }
		PHE { return F }
		LYS { return K }
		ASN { return N }
		GLN { return Q }
		ARG { return R }
		TRP { return W }
		TYR { return Y }
	}
	return [string index $upper 0]
}

######################################################################
#
# get prefix portion of a filename using sep as a character separator 
#
proc u_file_prefix {name sep} {
	set fn [ file tail $name ]
	set ix [ string last $sep $fn ]
	if {$ix == -1} {
		return $fn
	} else {
		return [string range $fn 0 [expr $ix-1]]
	}
}

######################################################################
#
# get suffix portion of a filename using sep as a character separator 
#
proc u_file_suffix {name sep} {
	set fn [ file tail $name ]
	set ix [ string last $sep $fn ]
	if {$ix == -1} {
		return $fn
	} else {
		return [string range $fn [expr $ix+1] end]
	}
}

######################################################################
#
# see if file1 is newer than file2 
#
proc u_file_newer {file1 file2} {
	if ![file exists $file2] {
		return 1
	} else {
		expr [file mtime $file1] > [file mtime $file2]
	}
}

######################################################################
#
# sort first number in a list
#
proc u_list_sort {a b} {
    set x [lindex $a 0]
	set y [lindex $b 0]
	if { $x < $y } {return 0 }
	return 1
}

######################################################################
#
# delete element in a list by value
#
proc u_list_delete {list value} {
	set ix [lsearch -exact $list $value]
	if { $ix >= 0 } {
		return [lreplace $list $ix $ix]
	} else {
		return $list
	}
}
