
########################################################################
#
# This is the front end for the Xalign program.  
#
# Tim Jellard 
# June 1996
#
# Robert Boyko
# Feb 2004 - completely revised
#
# usage: call xalign_main with installation directory and workspace directory. 
#
########################################################################
 
########################################################################
#
# main xalign startup script 
#
proc xalign_main { installdir workdir } {
	global varXalign 

	# check our arguments
	if { [ catch { set varXalign(INSTALL_DIR) $installdir } err] } {
		error "Usage: xalign_main installdir workdir"
	}
	if { [ catch { set varXalign(WORK_DIR) $workdir } err] } {
		error "Usage: xalign_main installdir workdir"
	}
	if ![ file isdirectory $varXalign(INSTALL_DIR) ] {
		error "Cannot find xalign installation directory '$varXalign(INSTALL_DIR)'."
	}
	if ![ file isdirectory $varXalign(WORK_DIR) ] {
		error "Cannot find workspace directory '$varXalign(WORK_DIR)'."
	}

	# setup our environment
	init_xalign

	# read defaults unless we are in smartnotebook
	read_xalign_defaults $varXalign(file,DEFAULTS) $varXalign(SYS_DEFAULTS)
	set_xalign_parms

	# define main window
	set f $varXalign(MAIN_WINDOW)
	toplevel $f
	wm geometry $f $varXalign(geometry,main) 
	wm title $f "xalign - $varXalign(INSTALL_DIR)"
	set f $varXalign(MAIN_WINDOW).inp
	frame $f
	pack $f
	panel_xalign_menubar $f

	# draw contents of first window
	panel_xalign_startup $f.win
	tkwait window $f
}

########################################################################
#
# init variables
#
proc init_xalign { } {
	global varXalign 

	set varXalign(MAIN_WINDOW)	".xalign"
	set varXalign(WEB_HOME)		"http://www.pence.ca/software/xalign"

	# location of binary files called by this tcl script
	set varXalign(BIN_DIR)  		"$varXalign(INSTALL_DIR)/bin"
	set varXalign(XALIGN_PROG) 		"$varXalign(BIN_DIR)/xalign_x"

	# location of lib files needed by xalign
	set varXalign(LIB_DIR) 			"$varXalign(INSTALL_DIR)/lib/xalign"
	set varXalign(SYS_DEFAULTS) 	"$varXalign(LIB_DIR)/xalign.defaults"
	set varXalign(SYS_PARMS)		"$varXalign(LIB_DIR)/xalign.parms"

	set varXalign(PARMS_KEY)		">>"

	set varXalign(DOC,AUTHORS)		"$varXalign(LIB_DIR)/doc.authors"
	set varXalign(DOC,BASIC)		"$varXalign(LIB_DIR)/doc.basic"
	set varXalign(DOC,RESULTS)		"$varXalign(LIB_DIR)/doc.results"
	set varXalign(DOC,INPUT1)		"$varXalign(LIB_DIR)/doc.input1"
	set varXalign(DOC,INPUT2)		"$varXalign(LIB_DIR)/doc.input2"
	set varXalign(DOC,INPUT3)		"$varXalign(LIB_DIR)/doc.input3"
	set varXalign(DOC,INPUT4)		"$varXalign(LIB_DIR)/doc.input4"
	set varXalign(DOC,OUTPUT1)		"$varXalign(LIB_DIR)/doc.output1"
	set varXalign(DOC,OUTPUT2)		"$varXalign(LIB_DIR)/doc.output2"

	set varXalign(geometry,main)	"750x500+100+100"
	set varXalign(geometry,view)	"+150+150"
	set varXalign(geometry,result)	"+200+200"
	set varXalign(padx)				15

	# Input Files created by xalign
	set varXalign(file,DEFAULTS) 	"$varXalign(WORK_DIR)/xalign.defaults"
	set varXalign(file,PARMS)		"$varXalign(WORK_DIR)/xalign.parms"
	set varXalign(file,RUN)			"$varXalign(WORK_DIR)/xalign.run"

	set varXalign(out,align) 		"$varXalign(WORK_DIR)/xalign.out"
	set varXalign(out,log) 			"$varXalign(WORK_DIR)/xalign.log"

	set varXalign(file,input)	 	""
	set varXalign(file,seq)		 	""
	set varXalign(file,ppm)			"" 

	set dirList [glob -nocomplain ./*]
	foreach dir $dirList {
		if [file isdirectory $dir] {
			set fn "[glob -nocomplain $dir/*.ppm] [glob -nocomplain $dir/*.PPM]"
			puts stdout "fn = $fn"
			if { [string length $fn] > 0 } {
				set varXalign(file,ppm) $dir
				break;
			}
		}
	}
	set fn [glob -nocomplain ./*.seq]
	if { [string length $fn] > 0 } {
		set varXalign(file,seq) [lindex $fn 0]
	}
}

###############################################################################
#
# create user's xalign.parms file 
#
proc set_xalign_parms { } {
	global varXalign 

	set fParm "XALIGN:SCOR_MATRIX"

	# file already exists, just read the matrix file name 
	if [file readable $varXalign(file,PARMS)] {
		set infp [open $varXalign(file,PARMS) r ]
		while {[gets $infp line] >= 0} {
			set nd [scan $line "%s %s" key value]
			if { $nd == 2 && $key == $fParm } {
				set varXalign(file,MATRIX) $value
				break
			}
		}
		close $infp
		return
	}

	if [catch {open $varXalign(file,PARMS) w} outfp] {
		error "Cannot open user parameter file: '$varXalign(file,PARMS)'"
	}

	# we must read system parms and write copy for the user
	if [catch {open $varXalign(SYS_PARMS) r} infp] {
		error "Cannot open parameter file: '$varXalign(SYS_PARMS)'"
	}

	while {[gets $infp line] >= 0} {
		set nd [scan $line "%s %s" key value]
		if { $nd == 2 && $key == $fParm } {
			set varXalign(file,MATRIX) $varXalign(LIB_DIR)/$value
			puts $outfp "$key $varXalign(file,MATRIX)"
		} else {
			puts $outfp $line
		}
	}
	close $infp
	close $outfp
}

###############################################################################
###############################################################################

###############################################################################
#
# main xalign window
#
proc panel_xalign_startup { f } {
	global varXalign 

	pack [frame $f]

	label $f.titleLabel -text "Welcome to xalign!" 
	pack $f.titleLabel
	message $f.intro -text \
		"This software allows you to create a multiple sequence alignment\
		for a number of protein sequences. If\
		you are unfamiliar with the input\
		file format, please click the HELP menu item for format\
		specification and examples.\n\nThe software is capable of generating\
		a default input file from a directory of PPM formatted chemical\
		shift files and a sequence file. Just select the second button,\
		enter or verify the required input files\
		and press CALCULATE.
		" 
	pack $f.intro
	frame $f.select
	pack $f.select
	radiobutton $f.select.0 \
		-text "I have my own xalign input file ready.   " \
		-variable varXalign(inpMode) -value 0 \
		-command "xalign_select $f"
	radiobutton $f.select.1 \
		-text "Please generate a default input file   " \
		-variable varXalign(inpMode) -value 1 \
		-command "xalign_select $f"
	pack $f.select.0 $f.select.1 -side top -anchor w -pady 2

	pack [frame $f.file] -pady 15

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20 
	button $f.buttons.exe
    button $f.buttons.quit -text Exit \
		-padx $varXalign(padx) \
		-command "exit_xalign"
    pack $f.buttons.exe $f.buttons.quit -side left

	set varXalign(inpMode) 0
	$f.select.0 invoke
}

###############################################################################
#
# read xalign input file, go on to output panel if ok 
#
proc xalign_select { f } {
	global varXalign

	destroy $f.file.ent
	destroy $f.file.1
	destroy $f.file.2
	if {$varXalign(inpMode) == 0} {
		x_panel_entry $f.file.ent       "Input File:" \
			18 varXalign(file,input) 40 { } 1 1
		$f.buttons.exe configure -text "Continue" -command "do_xalign_input $f"
	} else {
		x_panel_entry $f.file.1        "Sequence File:" \
			18 varXalign(file,seq) 40 { } 1 1
		x_panel_entry $f.file.2       "PPM Directory:" \
			18 varXalign(file,ppm) 40 { } 2 0
		$f.buttons.exe configure -text "Calculate" \
			-padx $varXalign(padx) \
			-command "make_xalign_input $f"
	}
}

###############################################################################
#
# make xalign input file from directory of PPM formatted chemical shifts
#
proc make_xalign_input { f } {
	global varXalign

	if [ catch { set fp [open $varXalign(file,seq) r ] } err] {
		error "Cannot open sequence file ' $varXalign(file,seq)'."
	}
	set varXalign(file,input) "./xalign.input"
	if [ catch { set outfp [open $varXalign(file,input) w ] } err] {
		error "Cannot write to xalign input file ' $varXalign(file,input)'."
	}
	puts $outfp "#"
	puts $outfp "# This xalign input file generated automatically by software"
	puts $outfp "# using the ppm formatted files in '$varXalign(file,ppm)'."
	puts $outfp "#"

	set seq ""
	set root [file tail $varXalign(file,seq)]
	set root [file rootname $root]
	set root [string range $root 0 7]
	while {[gets $fp r3] >= 0} {
		set r1 [x_resname_3to1 $r3]
		set seq "$seq$r1"
	}
	if { [string length $seq] > 0 } {
		puts $outfp ">$root $varXalign(file,input)"
		puts $outfp "$seq"
	}

	if ![file isdirectory $varXalign(file,ppm)] {
		error "Cannot open directory ' $varXalign(file,ppm)'."
	}
	set input [glob -nocomplain $varXalign(file,ppm)/*]
	foreach fn $input {
		if ![ catch { set fp [open $fn r ] } err] {
			set prev -99
			set seq ""
			set root [file tail $fn]
			set root [file rootname $root]
			set root [string range $root 0 7]
			while {[gets $fp line] >= 0} {
				if [ regexp {^[0-9]+:(.*)_([0-9]+):.*$} $line aStr r3 num ] {
					if {$num != $prev} {
						if { $prev == -99 } {
							set lo $num
						}
						set prev $num
						set r1 [x_resname_3to1 $r3]
						set seq "$seq$r1"
					}
				}
			}
			if { [string length $seq] > 0 } {
				set open {[}
				set close {]}
				puts $outfp ">$root $fn $open$lo-$num$close"
				puts $outfp "$seq"
			}
		}
	}
	close $outfp
	$f.select.0 invoke
}

######################################################################
#
# convert amino acid 3 letter code to 1 letter code
#
proc x_resname_3to1 { name3 } {
	set upper [string toupper $name3]
	switch -regexp $upper {
		ASP { return D }
		GLU { return E }
		PHE { return F }
		LYS { return K }
		ASN { return N }
		GLN { return Q }
		ARG { return R }
		TRP { return W }
		TYR { return Y }
	}
	return [string index $upper 0]
}

###############################################################################
#
# read xalign input file, go on to output panel if ok 
#
proc do_xalign_input { f } {
	global varXalign seqDat

	# check input file
	if { [ catch { set fp [open $varXalign(file,input) r ] } err] } {
		error "Cannot read data input file $varXalign(file,input)."
	}
 
	# count number of sequences in input file
	set seqDat(seqNum) 0
	while {[gets $fp line] >= 0} {
   		set aa [string index $line 0]
   		if { $aa == ">" } {
		   set seqDat(line,$seqDat(seqNum)) $line
   		   incr seqDat(seqNum)
   		}
	}
	close $fp
 
	# need at least 2 sequences.
	if { $seqDat(seqNum) < 2 }  {
		error "I did not find at least two sequences to align, are you\
		sure this file is in the correct format?"
	}

	destroy $f
	panel_xalign_output $f
}

###############################################################################
#
# xalign output and execute panel
#
proc panel_xalign_output { f } {
	global varXalign 

	pack [frame $f]

	label $f.titleLabel -text "Output Specification for xalign" 
	pack $f.titleLabel
	message $f.intro -text \
		"Usually it is best to let the xalign software determine the\
		order of the sequences in the multiple alignment. Press the\
		the CALCULATE button and examine the results. If you are not satisfied\
		with the quality of the output, select the 'Anchoring and \
		Clustering' HELP option \
		for hints on obtaining correct results."
	pack $f.intro

	frame $f.select
	pack $f.select
	radiobutton $f.select.0 \
		-text "Let software decide ordering of sequences   " \
		-variable varXalign(aMode) -value 0 
	radiobutton $f.select.1 \
		-text "Align sequences in input order   " \
		-variable varXalign(aMode) -value 1 
	radiobutton $f.select.2 \
		-text "Align sequences to one selected protein   " \
		-variable varXalign(aMode) -value 2 -command panel_xalign_seq_select
	pack $f.select.0 $f.select.1 $f.select.2 -side top -anchor w -pady 2
	$f.select.0 invoke

	x_panel_entry $f.output       "Output File:" \
		12 varXalign(out,align) 50 { } 0 0
	pack $f.output -pady 20

	# buttons section
	frame $f.buttons
	pack $f.buttons -pady 20 
	button $f.buttons.cont -text "Calculate" \
		-padx $varXalign(padx) \
		-command "xalign_execute"
    button $f.buttons.quit -text Exit \
		-padx $varXalign(padx) \
		-command "exit_xalign"
    pack $f.buttons.cont $f.buttons.quit -side left
}

###############################################################################
#
# executes xalign c program
#
proc xalign_execute { } {
	global varXalign seqDat

	# make sure output file is writeable
	if { [ catch { set fp [open $varXalign(out,align) w ] } err] } {
		error "Illegal output file '$varXalign(out,align)'."
	}

	# decide how to do the alignment 
	if {$varXalign(aMode) == 0} { set curSelect [expr 1 + $seqDat(seqNum) ] }
	if {$varXalign(aMode) == 1} { set curSelect [expr 2 + $seqDat(seqNum) ] }
	if {$varXalign(aMode) == 2} { 
		if ![info exists varXalign(seqSelect)] {
			error "Protein was not selected"
		}
		set curSelect $varXalign(seqSelect)
	}

	# make input run file for xalign_x 
	set fp_run [open $varXalign(file,RUN) w]
	puts $fp_run 1
	puts $fp_run $varXalign(file,input)
	puts $fp_run $curSelect
	puts $fp_run $varXalign(out,align)
	close $fp_run
 
	# call xalign c program, check log file
	puts stdout "prog = $varXalign(XALIGN_PROG) -i $varXalign(file,PARMS) < $varXalign(file,RUN)"
	set unixcmd "$varXalign(XALIGN_PROG) -i $varXalign(file,PARMS) < $varXalign(file,RUN)"
	if [ catch { eval exec $unixcmd > $varXalign(out,log)} err] {
		if { [file size $varXalign(out,log)] > 0 } {
			x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(out,log)
		} else {
			error "Problems running xalign_x c program. $err"
		}
	} else {
		x_panel_show_results "$varXalign(out,align) $varXalign(out,log)"
	}
}

###############################################################################
# 
# show window for sequence selection
#
proc panel_xalign_seq_select { } {
	global varXalign seqDat

	destroy varXalign(seqSelect)
	set f $varXalign(MAIN_WINDOW).seq
	toplevel $f
	wm title $f "xalign - sequence select"

	label $f.titleLabel -text "Sequence Selection" 
	pack $f.titleLabel
	message $f.intro -text \
		"By clicking on a sequence in the listbox below, you are telling\
		the xalign software to use this as the first sequence. Each of\
		the remaining sequences are added to the multiple alignment one\
		by one from most to least homologous. "
	pack $f.intro

	frame $f.fr0 
	pack $f.fr0 -pady 20 -padx 20

	set seqList [listbox $f.fr0.listing \
			-setgrid false -selectmode single \
			-xscrollcommand "$f.fr0.xscroll set" \
			-yscrollcommand "$f.fr0.yscroll set" \
			-relief raised -bd 2 -width 50 ]
	scrollbar $f.fr0.xscroll -orient horizontal -command "$f.fr0.listing xview"
	scrollbar $f.fr0.yscroll -command "$f.fr0.listing yview"
	pack $f.fr0.xscroll -side bottom -fill x
	pack $f.fr0.yscroll -side right -fill y
	pack $f.fr0.listing -side left -fill both -expand true

	bind $seqList <ButtonRelease-1> \
			[list bind_xalign_select $seqList %y]

	$seqList delete 0 end
	for {set i 0} {$i < $seqDat(seqNum)} {incr i} {
		$seqList insert end $seqDat(line,$i)
	}

	button $f.quit -text "Dismiss" \
		-padx $varXalign(padx) \
		-command "destroy $f"
	pack $f.quit -pady 5
}

###############################################################################
#
# select sequence in listbox
#
proc bind_xalign_select { list y } {
	global varXalign
	set varXalign(seqSelect) [expr 1 + [$list nearest $y]]
}

########################################################################
#
# main xalign shutdown routine 
#
proc exit_xalign {} {
	global varXalign 
	destroy $varXalign(MAIN_WINDOW)
	exec rm -f $varXalign(file,RUN) xalign.parms.tmp $varXalign(out,log)
}

########################################################################
#
# read the defaults file (look in user's directory and system location)
#
proc read_xalign_defaults { f_user f_sys } {

	# first read system defaults
	if [file readable $f_sys ] {
		if [catch {option readfile $f_sys interactive} err] {
			puts stderr "Problem reading option in $f_sys: $err"
			exit
		}
	}

	# then read user defaults
	if [file readable $f_user ] {
		if [catch {option readfile $f_user interactive} err] {
			puts stderr "Warning, problem reading option in $f_user: $err"
		}
	}
}

########################################################################
#
# create menubar
#
proc panel_xalign_menubar { f } {
	set mb $f.mbar
	frame $mb -relief raised -bd 2
	frame $f.dummy
	pack $mb $f -side top -fill x
	panel_xalign_menus $mb
}

#
# create menu items
#
proc panel_xalign_menus {f} {
	global varXalign

	menubutton $f.file -padx 10 -text "File" -underline 0 -menu $f.file.menu
	menubutton $f.view -text "Help" -underline 0 -menu $f.view.menu
	
	menu $f.file.menu
	$f.file.menu add command -label "About this software..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,AUTHORS)"
	$f.file.menu add separator
	$f.file.menu add command -label "Exit" -command "exit_xalign" 

	menu $f.view.menu
	$f.view.menu add separator
	$f.view.menu add command -label "Input file format..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,BASIC)"
	$f.view.menu add separator
	$f.view.menu add command -label "Input file example 1..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,INPUT1)"
	$f.view.menu add command -label "Input file example 2..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,INPUT2)"
	$f.view.menu add command -label "Input file example 3..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,INPUT3)"
	$f.view.menu add command -label "Input file example 4..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,INPUT4)"
	$f.view.menu add separator
	$f.view.menu add command -label "Output example 1..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,OUTPUT1)"
	$f.view.menu add command -label "Output example 2..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,OUTPUT2)"
	$f.view.menu add command -label "Anchoring and Clustering..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(DOC,RESULTS)"
	$f.view.menu add separator
	$f.view.menu add command -label "User parameters..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(file,PARMS)"
	$f.view.menu add command -label "Basic alignment parameters..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(file,MATRIX)"
	$f.view.menu add command -label "Gui defaults..." \
		-command "x_panel_file_view $varXalign(MAIN_WINDOW).view varXalign(SYS_DEFAULTS)"
	
	pack $f.file $f.view -side left
	tk_menuBar $f $f.file $f.view
	focus $f
}

########################################################################
########################################################################

########################################################################
#
# generic xalign entry widget
#
proc x_panel_entry { f title tWidth tVar eWidth cmd browse view } {
	global varXalign

    pack [frame $f] -fill x -expand true -padx 20
	pack [frame $f.a] -side left
	pack [frame $f.b] -side left -fill x -expand true
    pack [frame $f.c] -side left
    label $f.a.label -text $title -anchor e -width $tWidth
    pack $f.a.label -side right -padx 5
    entry $f.b.entry -bd 2 -textvariable $tVar -width $eWidth -relief sunken
	bind $f.b.entry <Return> $cmd
	pack $f.b.entry -side left -fill x -expand true
	if { $browse > 0 } {
		if { $browse > 1 } {
			button $f.c.browse -text Browse \
				-padx $varXalign(padx) \
				-command "x_panel_dir_browse $tVar"
			pack $f.c.browse -side left -padx 8
		} else {
			button $f.c.browse -text Browse \
				-padx $varXalign(padx) \
				-command "x_panel_file_browse $tVar"
			pack $f.c.browse -side left -padx 8
		}
	}
	if { $view > 0 } {
		button $f.c.view -text View \
			-padx $varXalign(padx) \
			-command "x_panel_file_view $varXalign(MAIN_WINDOW).view $tVar"
		pack $f.c.view -side left
	}
}

#
# this routine creates a window to display a file 
#
proc x_panel_file_view {f fVar} {
	global varXalign

	upvar $fVar fn
	if [catch {open $fn r} fp] {
		error "Cannot read file: $fn"
	}

	if ![winfo exists $f] {
		toplevel $f 
		wm geometry $f $varXalign(geometry,view)
	} else {
		raise $f
		destroy $f.results $f.quit
	}

	wm title $f "View file:  $fn"

	x_panel_textwin $f.results 80 30
	pack $f.results -pady 10 -side top -fill both -expand true

	button $f.quit -text "Dismiss" \
		-padx $varXalign(padx) \
		-command "destroy $f"
	pack $f.quit -pady 5
	x_load_file $fn $f.results.text
}

#
# purpose: to call a tk routine to navigate user's file system 
#
proc x_panel_file_browse { fn } {
	global varXalign 
	set chosen [tk_getOpenFile]
	if { $chosen != "" } { 
		set currentpwd [pwd] 
		set cmd "regsub {$currentpwd\/} $chosen \"\" shortchosen"
		eval $cmd
		set $fn $shortchosen
	}
}

#
# purpose: to call a tk routine to navigate user's file system 
#
proc x_panel_dir_browse { dir } {
	global varXalign 
	set chosen [tk_chooseDirectory]
	if { $chosen != "" } { 
		set currentpwd [pwd] 
		set cmd "regsub {$currentpwd\/} $chosen \"\" shortchosen"
		eval $cmd
		set $dir $shortchosen
	}
}

#
# display xalign results
#
proc x_panel_show_results { files } {
    global varXalign

	set f $varXalign(MAIN_WINDOW).results
	if ![winfo exists $f] {
		toplevel $f 
		wm geometry $f $varXalign(geometry,result)
		wm title $f "results"
		set varXalign(results,select) 0
	} else {
		raise $f
		destroy $f.fr1 $f.results $f.quit
	}

	frame $f.fr1
	pack $f.fr1 -pady 20
	
	set count 0
	foreach fn $files {
		set name [file tail $fn]
		radiobutton $f.fr1.$count -text "$name   " -value $count \
			-variable varXalign(results,select)
		pack $f.fr1.$count -side left
		bind $f.fr1.$count <Button-1> "x_load_file $fn $f.results.text"
		incr count
	}

	x_panel_textwin $f.results 80 30
	set fn [ lindex $files $varXalign(results,select) ]
	x_load_file $fn $f.results.text 
	$f.fr1.$varXalign(results,select) invoke

	pack $f.results -pady 10 -side top -fill both -expand true
	button $f.quit -text "Dismiss" \
		-padx $varXalign(padx) \
		-command "destroy $f"
	pack $f.quit -pady 5
}

#   
# create a generic widget for file text display
#       
proc x_panel_textwin { f width height } {
	frame $f
	pack $f -pady 10 -side top -fill both -expand true
    text $f.text -relief raised -bd 2 \
		-width $width -height $height \
		-setgrid true -wrap none \
		-xscrollcommand [list $f.xscroll set] \
		-yscrollcommand [list $f.yscroll set]
	scrollbar $f.xscroll -orient horizontal \
		-command [list $f.text xview]
	scrollbar $f.yscroll -orient vertical \
		-command [list $f.text yview]
	pack $f.xscroll -side bottom -fill x
	pack $f.yscroll -side right -fill y
	pack $f.text -side left -fill both -expand true
}

#
# loads a file onto a canvas - code taken form "TCL and the TK toolkit"
#
proc x_load_file { fname wind } {
	$wind delete 1.0 end
	if [catch {open $fname r} fp] {
		error "Cannot read file: $fname"
	}
	while {![eof $fp]} {
   		$wind insert end [read $fp 10000]
	}
	close $fp
}
